import { inspect as e } from "../jsutils/inspect.mjs";

import { memoize3 as r } from "../jsutils/memoize3.mjs";

import { invariant as t } from "../jsutils/invariant.mjs";

import { isPromise as n } from "../jsutils/isPromise.mjs";

import { isObjectLike as i } from "../jsutils/isObjectLike.mjs";

import { promiseReduce as o } from "../jsutils/promiseReduce.mjs";

import { promiseForObject as a } from "../jsutils/promiseForObject.mjs";

import { pathToArray as u, addPath as l } from "../jsutils/Path.mjs";

import { isIterableObject as s } from "../jsutils/isIterableObject.mjs";

import { GraphQLError as f } from "../error/GraphQLError.mjs";

import { locatedError as c } from "../error/locatedError.mjs";

import { Kind as m } from "../language/kinds.mjs";

import { assertValidSchema as p } from "../type/validate.mjs";

import { SchemaMetaFieldDef as v, TypeMetaFieldDef as d, TypeNameMetaFieldDef as h } from "../type/introspection.mjs";

import { isNonNullType as y, isListType as b, isLeafType as j, isAbstractType as w, isObjectType as x } from "../type/definition.mjs";

import { getOperationRootType as V } from "../utilities/getOperationRootType.mjs";

import { getVariableValues as T, getArgumentValues as g } from "./values.mjs";

import { collectFields as E } from "./collectFields.mjs";

function execute(e) {
  var r = e.schema;
  var t = e.document;
  var i = e.rootValue;
  var a = e.contextValue;
  var u = e.variableValues;
  var s = e.operationName;
  var f = e.fieldResolver;
  var c = e.typeResolver;
  assertValidExecutionArguments(r);
  var m = buildExecutionContext(r, t, i, a, u, s, f, c);
  if (!("schema" in m)) {
    return {
      errors: m
    };
  }
  var p = function executeOperation(e, r, t) {
    var i = V(e.schema, r);
    var a = E(e.schema, e.fragments, e.variableValues, i, r.selectionSet, new Map, new Set);
    var u = void 0;
    try {
      var s = "mutation" === r.operation ? function executeFieldsSerially(e, r, t, i, a) {
        return o(a.entries(), (function(o, a) {
          var u = a[0];
          var s = a[1];
          var f = l(i, u, r.name);
          var c = executeField(e, r, t, s, f);
          if (void 0 === c) {
            return o;
          }
          if (n(c)) {
            return c.then((function(e) {
              o[u] = e;
              return o;
            }));
          }
          o[u] = c;
          return o;
        }), Object.create(null));
      }(e, i, t, u, a) : executeFields(e, i, t, u, a);
      if (n(s)) {
        return s.then(void 0, (function(r) {
          e.errors.push(r);
          return Promise.resolve(null);
        }));
      }
      return s;
    } catch (r) {
      e.errors.push(r);
      return null;
    }
  }(m, m.operation, i);
  return buildResponse(m, p);
}

function executeSync(e) {
  var r = execute(e);
  if (n(r)) {
    throw new Error("GraphQL execution failed to complete synchronously.");
  }
  return r;
}

function buildResponse(e, r) {
  if (n(r)) {
    return r.then((function(r) {
      return buildResponse(e, r);
    }));
  }
  return 0 === e.errors.length ? {
    data: r
  } : {
    errors: e.errors,
    data: r
  };
}

function assertValidExecutionArguments(e, r, t) {
  p(e);
}

function buildExecutionContext(e, r, t, n, i, o, a, u) {
  var l, s;
  var c;
  var p = Object.create(null);
  for (var v = 0, d = r.definitions; v < d.length; v += 1) {
    var h = d[v];
    switch (h.kind) {
     case m.OPERATION_DEFINITION:
      if (null == o) {
        if (void 0 !== c) {
          return [ new f("Must provide operation name if query contains multiple operations.") ];
        }
        c = h;
      } else if ((null === (l = h.name) || void 0 === l ? void 0 : l.value) === o) {
        c = h;
      }
      break;

     case m.FRAGMENT_DEFINITION:
      p[h.name.value] = h;
    }
  }
  if (!c) {
    if (null != o) {
      return [ new f('Unknown operation named "' + o + '".') ];
    }
    return [ new f("Must provide an operation.") ];
  }
  var y = null !== (s = c.variableDefinitions) && void 0 !== s ? s : [];
  var b = T(e, y, null != i ? i : {}, {
    maxErrors: 50
  });
  if (b.errors) {
    return b.errors;
  }
  return {
    schema: e,
    fragments: p,
    rootValue: t,
    contextValue: n,
    operation: c,
    variableValues: b.coerced,
    fieldResolver: null != a ? a : defaultFieldResolver,
    typeResolver: null != u ? u : defaultTypeResolver,
    errors: []
  };
}

function executeFields(e, r, t, i, o) {
  var u = Object.create(null);
  var s = !1;
  for (var f = 0, c = o.entries(); f < c.length; f += 1) {
    var m = c[f];
    var p = m[0];
    var v = executeField(e, r, t, m[1], l(i, p, r.name));
    if (void 0 !== v) {
      u[p] = v;
      if (n(v)) {
        s = !0;
      }
    }
  }
  if (!s) {
    return u;
  }
  return a(u);
}

function executeField(e, r, t, i, o) {
  var a;
  var l = getFieldDef(e.schema, r, i[0]);
  if (!l) {
    return;
  }
  var s = l.type;
  var f = null !== (a = l.resolve) && void 0 !== a ? a : e.fieldResolver;
  var m = buildResolveInfo(e, l, i, r, o);
  try {
    var p = f(t, g(l, i[0], e.variableValues), e.contextValue, m);
    var v;
    if (n(p)) {
      v = p.then((function(r) {
        return completeValue(e, s, i, m, o, r);
      }));
    } else {
      v = completeValue(e, s, i, m, o, p);
    }
    if (n(v)) {
      return v.then(void 0, (function(r) {
        return handleFieldError(c(r, i, u(o)), s, e);
      }));
    }
    return v;
  } catch (r) {
    return handleFieldError(c(r, i, u(o)), s, e);
  }
}

function buildResolveInfo(e, r, t, n, i) {
  return {
    fieldName: r.name,
    fieldNodes: t,
    returnType: r.type,
    parentType: n,
    path: i,
    schema: e.schema,
    fragments: e.fragments,
    rootValue: e.rootValue,
    operation: e.operation,
    variableValues: e.variableValues
  };
}

function handleFieldError(e, r, t) {
  if (y(r)) {
    throw e;
  }
  t.errors.push(e);
  return null;
}

function completeValue(r, i, o, a, m, p) {
  if (p instanceof Error) {
    throw p;
  }
  if (y(i)) {
    var v = completeValue(r, i.ofType, o, a, m, p);
    if (null === v) {
      throw new Error("Cannot return null for non-nullable field " + a.parentType.name + "." + a.fieldName + ".");
    }
    return v;
  }
  if (null == p) {
    return null;
  }
  if (b(i)) {
    return function completeListValue(e, r, t, i, o, a) {
      if (!s(a)) {
        throw new f('Expected Iterable, but did not find one for field "' + i.parentType.name + "." + i.fieldName + '".');
      }
      var m = r.ofType;
      var p = !1;
      var v = Array.from(a, (function(r, a) {
        var s = l(o, a, void 0);
        try {
          var f;
          if (n(r)) {
            f = r.then((function(r) {
              return completeValue(e, m, t, i, s, r);
            }));
          } else {
            f = completeValue(e, m, t, i, s, r);
          }
          if (n(f)) {
            p = !0;
            return f.then(void 0, (function(r) {
              return handleFieldError(c(r, t, u(s)), m, e);
            }));
          }
          return f;
        } catch (r) {
          return handleFieldError(c(r, t, u(s)), m, e);
        }
      }));
      return p ? Promise.all(v) : v;
    }(r, i, o, a, m, p);
  }
  if (j(i)) {
    return function completeLeafValue(r, t) {
      var n = r.serialize(t);
      if (void 0 === n) {
        throw new Error('Expected a value of type "' + e(r) + '" but received: ' + e(t));
      }
      return n;
    }(i, p);
  }
  if (w(i)) {
    return function completeAbstractValue(e, r, t, i, o, a) {
      var u;
      var l = (null !== (u = r.resolveType) && void 0 !== u ? u : e.typeResolver)(a, e.contextValue, i, r);
      if (n(l)) {
        return l.then((function(n) {
          return completeObjectValue(e, ensureValidRuntimeType(n, e, r, t, i, a), t, i, o, a);
        }));
      }
      return completeObjectValue(e, ensureValidRuntimeType(l, e, r, t, i, a), t, i, o, a);
    }(r, i, o, a, m, p);
  }
  if (x(i)) {
    return completeObjectValue(r, i, o, a, m, p);
  }
  t(!1);
}

function ensureValidRuntimeType(r, t, n, i, o, a) {
  if (null == r) {
    throw new f('Abstract type "' + n.name + '" must resolve to an Object type at runtime for field "' + o.parentType.name + "." + o.fieldName + '". Either the "' + n.name + '" type should provide a "resolveType" function or each possible type should provide an "isTypeOf" function.', i);
  }
  if (x(r)) {
    throw new f("Support for returning GraphQLObjectType from resolveType was removed in graphql-js@16.0.0 please return type name instead.");
  }
  if ("string" != typeof r) {
    throw new f('Abstract type "' + n.name + '" must resolve to an Object type at runtime for field "' + o.parentType.name + "." + o.fieldName + '" with value ' + e(a) + ', received "' + e(r) + '".');
  }
  var u = t.schema.getType(r);
  if (null == u) {
    throw new f('Abstract type "' + n.name + '" was resolved to a type "' + r + '" that does not exist inside the schema.', i);
  }
  if (!x(u)) {
    throw new f('Abstract type "' + n.name + '" was resolved to a non-object type "' + r + '".', i);
  }
  if (!t.schema.isSubType(n, u)) {
    throw new f('Runtime Object type "' + u.name + '" is not a possible type for "' + n.name + '".', i);
  }
  return u;
}

function completeObjectValue(e, r, t, i, o, a) {
  var u = O(e, r, t);
  if (r.isTypeOf) {
    var l = r.isTypeOf(a, e.contextValue, i);
    if (n(l)) {
      return l.then((function(n) {
        if (!n) {
          throw invalidReturnTypeError(r, a, t);
        }
        return executeFields(e, r, a, o, u);
      }));
    }
    if (!l) {
      throw invalidReturnTypeError(r, a, t);
    }
  }
  return executeFields(e, r, a, o, u);
}

function invalidReturnTypeError(r, t, n) {
  return new f('Expected value of type "' + r.name + '" but got: ' + e(t) + ".", n);
}

var O = r((function _collectSubfields(e, r, t) {
  var n = new Map;
  var i = new Set;
  for (var o = 0, a = t; o < a.length; o += 1) {
    var u = a[o];
    if (u.selectionSet) {
      n = E(e.schema, e.fragments, e.variableValues, r, u.selectionSet, n, i);
    }
  }
  return n;
}));

var defaultTypeResolver = function(e, r, t, o) {
  if (i(e) && "string" == typeof e.__typename) {
    return e.__typename;
  }
  var a = t.schema.getPossibleTypes(o);
  var u = [];
  for (var l = 0; l < a.length; l++) {
    var s = a[l];
    if (s.isTypeOf) {
      var f = s.isTypeOf(e, r, t);
      if (n(f)) {
        u[l] = f;
      } else if (f) {
        return s.name;
      }
    }
  }
  if (u.length) {
    return Promise.all(u).then((function(e) {
      for (var r = 0; r < e.length; r++) {
        if (e[r]) {
          return a[r].name;
        }
      }
    }));
  }
};

var defaultFieldResolver = function(e, r, t, n) {
  if (i(e) || "function" == typeof e) {
    var o = e[n.fieldName];
    if ("function" == typeof o) {
      return e[n.fieldName](r, t, n);
    }
    return o;
  }
};

function getFieldDef(e, r, t) {
  var n = t.name.value;
  if (n === v.name && e.getQueryType() === r) {
    return v;
  } else if (n === d.name && e.getQueryType() === r) {
    return d;
  } else if (n === h.name) {
    return h;
  }
  return r.getFields()[n];
}

export { assertValidExecutionArguments, buildExecutionContext, buildResolveInfo, defaultFieldResolver, defaultTypeResolver, execute, executeSync, getFieldDef };
