"use strict";

var e = require("../error/syntaxError.js");

var r = require("./ast.js");

var n = require("./tokenKind.js");

var t = require("./blockString.js");

var i = function Lexer(e) {
  var t = new r.Token(n.TokenKind.SOF, 0, 0, 0, 0);
  this.source = e;
  this.lastToken = t;
  this.token = t;
  this.line = 1;
  this.lineStart = 0;
};

i.prototype.advance = function advance() {
  this.lastToken = this.token;
  return this.token = this.lookahead();
};

i.prototype.lookahead = function lookahead() {
  var e = this.token;
  if (e.kind !== n.TokenKind.EOF) {
    do {
      if (e.next) {
        e = e.next;
      } else {
        var r = readNextToken(this, e.end);
        e.next = r;
        r.prev = e;
        e = r;
      }
    } while (e.kind === n.TokenKind.COMMENT);
  }
  return e;
};

function isSourceCharacter(e) {
  return e >= 32 || 9 === e || 10 === e || 13 === e;
}

function printCodePointAt(e, r) {
  var t = e.source.body;
  if (r >= t.length) {
    return n.TokenKind.EOF;
  }
  var i = t.charCodeAt(r);
  if (i >= 32 && i <= 126) {
    return 34 === i ? "'\"'" : '"' + t[r] + '"';
  }
  return "U+" + (i > 4095 ? "" : i > 255 ? "0" : i > 15 ? "00" : "000") + i.toString(16).toUpperCase();
}

function createToken(e, n, t, i, a) {
  return new r.Token(n, t, i, e.line, 1 + t - e.lineStart, a);
}

function readNextToken(r, t) {
  var i = r.source.body;
  var a = i.length;
  var o = t;
  while (o < a) {
    var c = i.charCodeAt(o);
    switch (c) {
     case 65279:
     case 9:
     case 32:
     case 44:
      ++o;
      continue;

     case 10:
      ++o;
      ++r.line;
      r.lineStart = o;
      continue;

     case 13:
      if (10 === i.charCodeAt(o + 1)) {
        o += 2;
      } else {
        ++o;
      }
      ++r.line;
      r.lineStart = o;
      continue;

     case 35:
      return readComment(r, o);

     case 33:
      return createToken(r, n.TokenKind.BANG, o, o + 1);

     case 36:
      return createToken(r, n.TokenKind.DOLLAR, o, o + 1);

     case 38:
      return createToken(r, n.TokenKind.AMP, o, o + 1);

     case 40:
      return createToken(r, n.TokenKind.PAREN_L, o, o + 1);

     case 41:
      return createToken(r, n.TokenKind.PAREN_R, o, o + 1);

     case 46:
      if (46 === i.charCodeAt(o + 1) && 46 === i.charCodeAt(o + 2)) {
        return createToken(r, n.TokenKind.SPREAD, o, o + 3);
      }
      break;

     case 58:
      return createToken(r, n.TokenKind.COLON, o, o + 1);

     case 61:
      return createToken(r, n.TokenKind.EQUALS, o, o + 1);

     case 64:
      return createToken(r, n.TokenKind.AT, o, o + 1);

     case 91:
      return createToken(r, n.TokenKind.BRACKET_L, o, o + 1);

     case 93:
      return createToken(r, n.TokenKind.BRACKET_R, o, o + 1);

     case 123:
      return createToken(r, n.TokenKind.BRACE_L, o, o + 1);

     case 124:
      return createToken(r, n.TokenKind.PIPE, o, o + 1);

     case 125:
      return createToken(r, n.TokenKind.BRACE_R, o, o + 1);

     case 34:
      if (34 === i.charCodeAt(o + 1) && 34 === i.charCodeAt(o + 2)) {
        return readBlockString(r, o);
      }
      return readString(r, o);
    }
    if (isDigit(c) || 45 === c) {
      return readNumber(r, o, c);
    }
    if (isNameStart(c)) {
      return readName(r, o);
    }
    throw e.syntaxError(r.source, o, 39 === c ? "Unexpected single quote character ('), did you mean to use a double quote (\")?" : isSourceCharacter(c) ? "Unexpected character: " + printCodePointAt(r, o) + "." : "Invalid character: " + printCodePointAt(r, o) + ".");
  }
  return createToken(r, n.TokenKind.EOF, a, a);
}

function readComment(e, r) {
  var t = e.source.body;
  var i = t.length;
  var a = r + 1;
  while (a < i) {
    var o = t.charCodeAt(a);
    if (10 === o || 13 === o) {
      break;
    }
    if (isSourceCharacter(o)) {
      ++a;
    } else {
      break;
    }
  }
  return createToken(e, n.TokenKind.COMMENT, r, a, t.slice(r + 1, a));
}

function readNumber(r, t, i) {
  var a = r.source.body;
  var o = t;
  var c = i;
  var d = !1;
  if (45 === c) {
    c = a.charCodeAt(++o);
  }
  if (48 === c) {
    if (isDigit(c = a.charCodeAt(++o))) {
      throw e.syntaxError(r.source, o, "Invalid number, unexpected digit after 0: " + printCodePointAt(r, o) + ".");
    }
  } else {
    o = readDigits(r, o, c);
    c = a.charCodeAt(o);
  }
  if (46 === c) {
    d = !0;
    c = a.charCodeAt(++o);
    o = readDigits(r, o, c);
    c = a.charCodeAt(o);
  }
  if (69 === c || 101 === c) {
    d = !0;
    if (43 === (c = a.charCodeAt(++o)) || 45 === c) {
      c = a.charCodeAt(++o);
    }
    o = readDigits(r, o, c);
    c = a.charCodeAt(o);
  }
  if (46 === c || isNameStart(c)) {
    throw e.syntaxError(r.source, o, "Invalid number, expected digit but got: " + printCodePointAt(r, o) + ".");
  }
  return createToken(r, d ? n.TokenKind.FLOAT : n.TokenKind.INT, t, o, a.slice(t, o));
}

function readDigits(r, n, t) {
  if (!isDigit(t)) {
    throw e.syntaxError(r.source, n, "Invalid number, expected digit but got: " + printCodePointAt(r, n) + ".");
  }
  var i = r.source.body;
  var a = n;
  var o = t;
  do {
    o = i.charCodeAt(++a);
  } while (isDigit(o));
  return a;
}

function readString(r, t) {
  var i = r.source.body;
  var a = i.length;
  var o = t + 1;
  var c = o;
  var d = "";
  while (o < a) {
    var s = i.charCodeAt(o);
    if (34 === s) {
      d += i.slice(c, o);
      return createToken(r, n.TokenKind.STRING, t, o + 1, d);
    }
    if (92 === s) {
      d += i.slice(c, o);
      var u = 117 === i.charCodeAt(o + 1) ? readEscapedUnicode(r, o) : readEscapedCharacter(r, o);
      d += u.value;
      c = o += u.size;
      continue;
    }
    if (10 === s || 13 === s) {
      break;
    }
    if (isSourceCharacter(s)) {
      ++o;
    } else {
      throw e.syntaxError(r.source, o, "Invalid character within String: " + printCodePointAt(r, o) + ".");
    }
  }
  throw e.syntaxError(r.source, o, "Unterminated string.");
}

function readEscapedUnicode(r, n) {
  var t = r.source.body;
  var i = function read16BitHexCode(e, r) {
    return readHexDigit(e.charCodeAt(r)) << 12 | readHexDigit(e.charCodeAt(r + 1)) << 8 | readHexDigit(e.charCodeAt(r + 2)) << 4 | readHexDigit(e.charCodeAt(r + 3));
  }(t, n + 2);
  if (i >= 0) {
    return {
      value: String.fromCharCode(i),
      size: 6
    };
  }
  throw e.syntaxError(r.source, n, 'Invalid Unicode escape sequence: "' + t.slice(n, n + 6) + '".');
}

function readHexDigit(e) {
  return e >= 48 && e <= 57 ? e - 48 : e >= 65 && e <= 70 ? e - 55 : e >= 97 && e <= 102 ? e - 87 : -1;
}

function readEscapedCharacter(r, n) {
  var t = r.source.body;
  switch (t.charCodeAt(n + 1)) {
   case 34:
    return {
      value: '"',
      size: 2
    };

   case 92:
    return {
      value: "\\",
      size: 2
    };

   case 47:
    return {
      value: "/",
      size: 2
    };

   case 98:
    return {
      value: "\b",
      size: 2
    };

   case 102:
    return {
      value: "\f",
      size: 2
    };

   case 110:
    return {
      value: "\n",
      size: 2
    };

   case 114:
    return {
      value: "\r",
      size: 2
    };

   case 116:
    return {
      value: "\t",
      size: 2
    };
  }
  throw e.syntaxError(r.source, n, 'Invalid character escape sequence: "' + t.slice(n, n + 2) + '".');
}

function readBlockString(r, i) {
  var a = r.source.body;
  var o = a.length;
  var c = i + 3;
  var d = c;
  var s = "";
  while (c < o) {
    var u = a.charCodeAt(c);
    if (34 === u && 34 === a.charCodeAt(c + 1) && 34 === a.charCodeAt(c + 2)) {
      s += a.slice(d, c);
      return createToken(r, n.TokenKind.BLOCK_STRING, i, c + 3, t.dedentBlockStringValue(s));
    }
    if (92 === u && 34 === a.charCodeAt(c + 1) && 34 === a.charCodeAt(c + 2) && 34 === a.charCodeAt(c + 3)) {
      s += a.slice(d, c) + '"""';
      d = c += 4;
      continue;
    }
    if (10 === u || 13 === u) {
      if (13 === u && 10 === a.charCodeAt(c + 1)) {
        c += 2;
      } else {
        ++c;
      }
      ++r.line;
      r.lineStart = c;
      continue;
    }
    if (isSourceCharacter(u)) {
      ++c;
    } else {
      throw e.syntaxError(r.source, c, "Invalid character within String: " + printCodePointAt(r, c) + ".");
    }
  }
  throw e.syntaxError(r.source, c, "Unterminated string.");
}

function readName(e, r) {
  var t = e.source.body;
  var i = t.length;
  var a = r + 1;
  while (a < i) {
    var o = t.charCodeAt(a);
    if (isLetter(o) || isDigit(o) || 95 === o) {
      ++a;
    } else {
      break;
    }
  }
  return createToken(e, n.TokenKind.NAME, r, a, t.slice(r, a));
}

function isNameStart(e) {
  return isLetter(e) || 95 === e;
}

function isDigit(e) {
  return e >= 48 && e <= 57;
}

function isLetter(e) {
  return e >= 97 && e <= 122 || e >= 65 && e <= 90;
}

exports.Lexer = i;

exports.isPunctuatorTokenKind = function isPunctuatorTokenKind(e) {
  return e === n.TokenKind.BANG || e === n.TokenKind.DOLLAR || e === n.TokenKind.AMP || e === n.TokenKind.PAREN_L || e === n.TokenKind.PAREN_R || e === n.TokenKind.SPREAD || e === n.TokenKind.COLON || e === n.TokenKind.EQUALS || e === n.TokenKind.AT || e === n.TokenKind.BRACKET_L || e === n.TokenKind.BRACKET_R || e === n.TokenKind.BRACE_L || e === n.TokenKind.PIPE || e === n.TokenKind.BRACE_R;
};
