import { syntaxError as e } from "../error/syntaxError.mjs";

import { Token as r } from "./ast.mjs";

import { TokenKind as t } from "./tokenKind.mjs";

import { dedentBlockStringValue as a } from "./blockString.mjs";

var n = function Lexer(e) {
  var a = new r(t.SOF, 0, 0, 0, 0);
  this.source = e;
  this.lastToken = a;
  this.token = a;
  this.line = 1;
  this.lineStart = 0;
};

n.prototype.advance = function advance() {
  this.lastToken = this.token;
  return this.token = this.lookahead();
};

n.prototype.lookahead = function lookahead() {
  var e = this.token;
  if (e.kind !== t.EOF) {
    do {
      if (e.next) {
        e = e.next;
      } else {
        var r = readNextToken(this, e.end);
        e.next = r;
        r.prev = e;
        e = r;
      }
    } while (e.kind === t.COMMENT);
  }
  return e;
};

function isPunctuatorTokenKind(e) {
  return e === t.BANG || e === t.DOLLAR || e === t.AMP || e === t.PAREN_L || e === t.PAREN_R || e === t.SPREAD || e === t.COLON || e === t.EQUALS || e === t.AT || e === t.BRACKET_L || e === t.BRACKET_R || e === t.BRACE_L || e === t.PIPE || e === t.BRACE_R;
}

function isSourceCharacter(e) {
  return e >= 32 || 9 === e || 10 === e || 13 === e;
}

function printCodePointAt(e, r) {
  var a = e.source.body;
  if (r >= a.length) {
    return t.EOF;
  }
  var n = a.charCodeAt(r);
  if (n >= 32 && n <= 126) {
    return 34 === n ? "'\"'" : '"' + a[r] + '"';
  }
  return "U+" + (n > 4095 ? "" : n > 255 ? "0" : n > 15 ? "00" : "000") + n.toString(16).toUpperCase();
}

function createToken(e, t, a, n, i) {
  return new r(t, a, n, e.line, 1 + a - e.lineStart, i);
}

function readNextToken(r, a) {
  var n = r.source.body;
  var i = n.length;
  var o = a;
  while (o < i) {
    var c = n.charCodeAt(o);
    switch (c) {
     case 65279:
     case 9:
     case 32:
     case 44:
      ++o;
      continue;

     case 10:
      ++o;
      ++r.line;
      r.lineStart = o;
      continue;

     case 13:
      if (10 === n.charCodeAt(o + 1)) {
        o += 2;
      } else {
        ++o;
      }
      ++r.line;
      r.lineStart = o;
      continue;

     case 35:
      return readComment(r, o);

     case 33:
      return createToken(r, t.BANG, o, o + 1);

     case 36:
      return createToken(r, t.DOLLAR, o, o + 1);

     case 38:
      return createToken(r, t.AMP, o, o + 1);

     case 40:
      return createToken(r, t.PAREN_L, o, o + 1);

     case 41:
      return createToken(r, t.PAREN_R, o, o + 1);

     case 46:
      if (46 === n.charCodeAt(o + 1) && 46 === n.charCodeAt(o + 2)) {
        return createToken(r, t.SPREAD, o, o + 3);
      }
      break;

     case 58:
      return createToken(r, t.COLON, o, o + 1);

     case 61:
      return createToken(r, t.EQUALS, o, o + 1);

     case 64:
      return createToken(r, t.AT, o, o + 1);

     case 91:
      return createToken(r, t.BRACKET_L, o, o + 1);

     case 93:
      return createToken(r, t.BRACKET_R, o, o + 1);

     case 123:
      return createToken(r, t.BRACE_L, o, o + 1);

     case 124:
      return createToken(r, t.PIPE, o, o + 1);

     case 125:
      return createToken(r, t.BRACE_R, o, o + 1);

     case 34:
      if (34 === n.charCodeAt(o + 1) && 34 === n.charCodeAt(o + 2)) {
        return readBlockString(r, o);
      }
      return readString(r, o);
    }
    if (isDigit(c) || 45 === c) {
      return readNumber(r, o, c);
    }
    if (isNameStart(c)) {
      return readName(r, o);
    }
    throw e(r.source, o, 39 === c ? "Unexpected single quote character ('), did you mean to use a double quote (\")?" : isSourceCharacter(c) ? "Unexpected character: " + printCodePointAt(r, o) + "." : "Invalid character: " + printCodePointAt(r, o) + ".");
  }
  return createToken(r, t.EOF, i, i);
}

function readComment(e, r) {
  var a = e.source.body;
  var n = a.length;
  var i = r + 1;
  while (i < n) {
    var o = a.charCodeAt(i);
    if (10 === o || 13 === o) {
      break;
    }
    if (isSourceCharacter(o)) {
      ++i;
    } else {
      break;
    }
  }
  return createToken(e, t.COMMENT, r, i, a.slice(r + 1, i));
}

function readNumber(r, a, n) {
  var i = r.source.body;
  var o = a;
  var c = n;
  var s = !1;
  if (45 === c) {
    c = i.charCodeAt(++o);
  }
  if (48 === c) {
    if (isDigit(c = i.charCodeAt(++o))) {
      throw e(r.source, o, "Invalid number, unexpected digit after 0: " + printCodePointAt(r, o) + ".");
    }
  } else {
    o = readDigits(r, o, c);
    c = i.charCodeAt(o);
  }
  if (46 === c) {
    s = !0;
    c = i.charCodeAt(++o);
    o = readDigits(r, o, c);
    c = i.charCodeAt(o);
  }
  if (69 === c || 101 === c) {
    s = !0;
    if (43 === (c = i.charCodeAt(++o)) || 45 === c) {
      c = i.charCodeAt(++o);
    }
    o = readDigits(r, o, c);
    c = i.charCodeAt(o);
  }
  if (46 === c || isNameStart(c)) {
    throw e(r.source, o, "Invalid number, expected digit but got: " + printCodePointAt(r, o) + ".");
  }
  return createToken(r, s ? t.FLOAT : t.INT, a, o, i.slice(a, o));
}

function readDigits(r, t, a) {
  if (!isDigit(a)) {
    throw e(r.source, t, "Invalid number, expected digit but got: " + printCodePointAt(r, t) + ".");
  }
  var n = r.source.body;
  var i = t;
  var o = a;
  do {
    o = n.charCodeAt(++i);
  } while (isDigit(o));
  return i;
}

function readString(r, a) {
  var n = r.source.body;
  var i = n.length;
  var o = a + 1;
  var c = o;
  var s = "";
  while (o < i) {
    var u = n.charCodeAt(o);
    if (34 === u) {
      s += n.slice(c, o);
      return createToken(r, t.STRING, a, o + 1, s);
    }
    if (92 === u) {
      s += n.slice(c, o);
      var d = 117 === n.charCodeAt(o + 1) ? readEscapedUnicode(r, o) : readEscapedCharacter(r, o);
      s += d.value;
      c = o += d.size;
      continue;
    }
    if (10 === u || 13 === u) {
      break;
    }
    if (isSourceCharacter(u)) {
      ++o;
    } else {
      throw e(r.source, o, "Invalid character within String: " + printCodePointAt(r, o) + ".");
    }
  }
  throw e(r.source, o, "Unterminated string.");
}

function readEscapedUnicode(r, t) {
  var a = r.source.body;
  var n = function read16BitHexCode(e, r) {
    return readHexDigit(e.charCodeAt(r)) << 12 | readHexDigit(e.charCodeAt(r + 1)) << 8 | readHexDigit(e.charCodeAt(r + 2)) << 4 | readHexDigit(e.charCodeAt(r + 3));
  }(a, t + 2);
  if (n >= 0) {
    return {
      value: String.fromCharCode(n),
      size: 6
    };
  }
  throw e(r.source, t, 'Invalid Unicode escape sequence: "' + a.slice(t, t + 6) + '".');
}

function readHexDigit(e) {
  return e >= 48 && e <= 57 ? e - 48 : e >= 65 && e <= 70 ? e - 55 : e >= 97 && e <= 102 ? e - 87 : -1;
}

function readEscapedCharacter(r, t) {
  var a = r.source.body;
  switch (a.charCodeAt(t + 1)) {
   case 34:
    return {
      value: '"',
      size: 2
    };

   case 92:
    return {
      value: "\\",
      size: 2
    };

   case 47:
    return {
      value: "/",
      size: 2
    };

   case 98:
    return {
      value: "\b",
      size: 2
    };

   case 102:
    return {
      value: "\f",
      size: 2
    };

   case 110:
    return {
      value: "\n",
      size: 2
    };

   case 114:
    return {
      value: "\r",
      size: 2
    };

   case 116:
    return {
      value: "\t",
      size: 2
    };
  }
  throw e(r.source, t, 'Invalid character escape sequence: "' + a.slice(t, t + 2) + '".');
}

function readBlockString(r, n) {
  var i = r.source.body;
  var o = i.length;
  var c = n + 3;
  var s = c;
  var u = "";
  while (c < o) {
    var d = i.charCodeAt(c);
    if (34 === d && 34 === i.charCodeAt(c + 1) && 34 === i.charCodeAt(c + 2)) {
      u += i.slice(s, c);
      return createToken(r, t.BLOCK_STRING, n, c + 3, a(u));
    }
    if (92 === d && 34 === i.charCodeAt(c + 1) && 34 === i.charCodeAt(c + 2) && 34 === i.charCodeAt(c + 3)) {
      u += i.slice(s, c) + '"""';
      s = c += 4;
      continue;
    }
    if (10 === d || 13 === d) {
      if (13 === d && 10 === i.charCodeAt(c + 1)) {
        c += 2;
      } else {
        ++c;
      }
      ++r.line;
      r.lineStart = c;
      continue;
    }
    if (isSourceCharacter(d)) {
      ++c;
    } else {
      throw e(r.source, c, "Invalid character within String: " + printCodePointAt(r, c) + ".");
    }
  }
  throw e(r.source, c, "Unterminated string.");
}

function readName(e, r) {
  var a = e.source.body;
  var n = a.length;
  var i = r + 1;
  while (i < n) {
    var o = a.charCodeAt(i);
    if (isLetter(o) || isDigit(o) || 95 === o) {
      ++i;
    } else {
      break;
    }
  }
  return createToken(e, t.NAME, r, i, a.slice(r, i));
}

function isNameStart(e) {
  return isLetter(e) || 95 === e;
}

function isDigit(e) {
  return e >= 48 && e <= 57;
}

function isLetter(e) {
  return e >= 97 && e <= 122 || e >= 65 && e <= 90;
}

export { n as Lexer, isPunctuatorTokenKind };
