import { inspect as e } from "../jsutils/inspect.mjs";

import { keyMap as t } from "../jsutils/keyMap.mjs";

import { mapValue as n } from "../jsutils/mapValue.mjs";

import { toObjMap as i } from "../jsutils/toObjMap.mjs";

import { keyValMap as s } from "../jsutils/keyValMap.mjs";

import { instanceOf as o } from "../jsutils/instanceOf.mjs";

import { didYouMean as r } from "../jsutils/didYouMean.mjs";

import { identityFunc as p } from "../jsutils/identityFunc.mjs";

import { suggestionList as a } from "../jsutils/suggestionList.mjs";

import { GraphQLError as u } from "../error/GraphQLError.mjs";

import { Kind as f } from "../language/kinds.mjs";

import { print as c } from "../language/printer.mjs";

import { valueFromASTUntyped as d } from "../utilities/valueFromASTUntyped.mjs";

function isType(e) {
  return isScalarType(e) || isObjectType(e) || isInterfaceType(e) || isUnionType(e) || isEnumType(e) || isInputObjectType(e) || isListType(e) || isNonNullType(e);
}

function assertType(t) {
  if (!isType(t)) {
    throw new Error("Expected " + e(t) + " to be a GraphQL type.");
  }
  return t;
}

function isScalarType(e) {
  return o(e, h);
}

function assertScalarType(t) {
  if (!isScalarType(t)) {
    throw new Error("Expected " + e(t) + " to be a GraphQL Scalar type.");
  }
  return t;
}

function isObjectType(e) {
  return o(e, T);
}

function assertObjectType(t) {
  if (!isObjectType(t)) {
    throw new Error("Expected " + e(t) + " to be a GraphQL Object type.");
  }
  return t;
}

function isInterfaceType(e) {
  return o(e, m);
}

function assertInterfaceType(t) {
  if (!isInterfaceType(t)) {
    throw new Error("Expected " + e(t) + " to be a GraphQL Interface type.");
  }
  return t;
}

function isUnionType(e) {
  return o(e, N);
}

function assertUnionType(t) {
  if (!isUnionType(t)) {
    throw new Error("Expected " + e(t) + " to be a GraphQL Union type.");
  }
  return t;
}

function isEnumType(e) {
  return o(e, g);
}

function assertEnumType(t) {
  if (!isEnumType(t)) {
    throw new Error("Expected " + e(t) + " to be a GraphQL Enum type.");
  }
  return t;
}

function isInputObjectType(e) {
  return o(e, v);
}

function assertInputObjectType(t) {
  if (!isInputObjectType(t)) {
    throw new Error("Expected " + e(t) + " to be a GraphQL Input Object type.");
  }
  return t;
}

function isListType(e) {
  return o(e, y);
}

function assertListType(t) {
  if (!isListType(t)) {
    throw new Error("Expected " + e(t) + " to be a GraphQL List type.");
  }
  return t;
}

function isNonNullType(e) {
  return o(e, l);
}

function assertNonNullType(t) {
  if (!isNonNullType(t)) {
    throw new Error("Expected " + e(t) + " to be a GraphQL Non-Null type.");
  }
  return t;
}

function isInputType(e) {
  return isScalarType(e) || isEnumType(e) || isInputObjectType(e) || isWrappingType(e) && isInputType(e.ofType);
}

function assertInputType(t) {
  if (!isInputType(t)) {
    throw new Error("Expected " + e(t) + " to be a GraphQL input type.");
  }
  return t;
}

function isOutputType(e) {
  return isScalarType(e) || isObjectType(e) || isInterfaceType(e) || isUnionType(e) || isEnumType(e) || isWrappingType(e) && isOutputType(e.ofType);
}

function assertOutputType(t) {
  if (!isOutputType(t)) {
    throw new Error("Expected " + e(t) + " to be a GraphQL output type.");
  }
  return t;
}

function isLeafType(e) {
  return isScalarType(e) || isEnumType(e);
}

function assertLeafType(t) {
  if (!isLeafType(t)) {
    throw new Error("Expected " + e(t) + " to be a GraphQL leaf type.");
  }
  return t;
}

function isCompositeType(e) {
  return isObjectType(e) || isInterfaceType(e) || isUnionType(e);
}

function isAbstractType(e) {
  return isInterfaceType(e) || isUnionType(e);
}

var y = function GraphQLList(e) {
  isType(e);
  this.ofType = e;
};

y.prototype.toString = function toString() {
  return "[" + String(this.ofType) + "]";
};

y.prototype.toJSON = function toJSON() {
  return this.toString();
};

var l = function GraphQLNonNull(e) {
  isNullableType(e);
  this.ofType = e;
};

l.prototype.toString = function toString() {
  return String(this.ofType) + "!";
};

l.prototype.toJSON = function toJSON() {
  return this.toString();
};

function isWrappingType(e) {
  return isListType(e) || isNonNullType(e);
}

function assertWrappingType(t) {
  if (!isWrappingType(t)) {
    throw new Error("Expected " + e(t) + " to be a GraphQL wrapping type.");
  }
  return t;
}

function isNullableType(e) {
  return isType(e) && !isNonNullType(e);
}

function assertNullableType(t) {
  if (!isNullableType(t)) {
    throw new Error("Expected " + e(t) + " to be a GraphQL nullable type.");
  }
  return t;
}

function getNullableType(e) {
  if (e) {
    return isNonNullType(e) ? e.ofType : e;
  }
}

function isNamedType(e) {
  return isScalarType(e) || isObjectType(e) || isInterfaceType(e) || isUnionType(e) || isEnumType(e) || isInputObjectType(e);
}

function assertNamedType(t) {
  if (!isNamedType(t)) {
    throw new Error("Expected " + e(t) + " to be a GraphQL named type.");
  }
  return t;
}

function getNamedType(e) {
  if (e) {
    var t = e;
    while (isWrappingType(t)) {
      t = t.ofType;
    }
    return t;
  }
}

function resolveReadonlyArrayThunk(e) {
  return "function" == typeof e ? e() : e;
}

function resolveObjMapThunk(e) {
  return "function" == typeof e ? e() : e;
}

var h = function GraphQLScalarType(e) {
  var t, n, s, o;
  var r = null !== (t = e.parseValue) && void 0 !== t ? t : p;
  this.name = e.name;
  this.description = e.description;
  this.specifiedByURL = e.specifiedByURL;
  this.serialize = null !== (n = e.serialize) && void 0 !== n ? n : p;
  this.parseValue = r;
  this.parseLiteral = null !== (s = e.parseLiteral) && void 0 !== s ? s : function(e, t) {
    return r(d(e, t));
  };
  this.extensions = e.extensions && i(e.extensions);
  this.astNode = e.astNode;
  this.extensionASTNodes = null !== (o = e.extensionASTNodes) && void 0 !== o ? o : [];
};

h.prototype.toConfig = function toConfig() {
  return {
    name: this.name,
    description: this.description,
    specifiedByURL: this.specifiedByURL,
    serialize: this.serialize,
    parseValue: this.parseValue,
    parseLiteral: this.parseLiteral,
    extensions: this.extensions,
    astNode: this.astNode,
    extensionASTNodes: this.extensionASTNodes
  };
};

h.prototype.toString = function toString() {
  return this.name;
};

h.prototype.toJSON = function toJSON() {
  return this.toString();
};

var T = function GraphQLObjectType(e) {
  var t;
  this.name = e.name;
  this.description = e.description;
  this.isTypeOf = e.isTypeOf;
  this.extensions = e.extensions && i(e.extensions);
  this.astNode = e.astNode;
  this.extensionASTNodes = null !== (t = e.extensionASTNodes) && void 0 !== t ? t : [];
  this._fields = function() {
    return defineFieldMap(e);
  };
  this._interfaces = function() {
    return defineInterfaces(e);
  };
};

T.prototype.getFields = function getFields() {
  if ("function" == typeof this._fields) {
    this._fields = this._fields();
  }
  return this._fields;
};

T.prototype.getInterfaces = function getInterfaces() {
  if ("function" == typeof this._interfaces) {
    this._interfaces = this._interfaces();
  }
  return this._interfaces;
};

T.prototype.toConfig = function toConfig() {
  return {
    name: this.name,
    description: this.description,
    interfaces: this.getInterfaces(),
    fields: fieldsToFieldsConfig(this.getFields()),
    isTypeOf: this.isTypeOf,
    extensions: this.extensions,
    astNode: this.astNode,
    extensionASTNodes: this.extensionASTNodes
  };
};

T.prototype.toString = function toString() {
  return this.name;
};

T.prototype.toJSON = function toJSON() {
  return this.toString();
};

function defineInterfaces(e) {
  var t;
  return resolveReadonlyArrayThunk(null !== (t = e.interfaces) && void 0 !== t ? t : []);
}

function defineFieldMap(e) {
  var t = resolveObjMapThunk(e.fields);
  return n(t, (function(e, t) {
    var n;
    var s = null !== (n = e.args) && void 0 !== n ? n : {};
    return {
      name: t,
      description: e.description,
      type: e.type,
      args: defineArguments(s),
      resolve: e.resolve,
      subscribe: e.subscribe,
      deprecationReason: e.deprecationReason,
      extensions: e.extensions && i(e.extensions),
      astNode: e.astNode
    };
  }));
}

function defineArguments(e) {
  return Object.entries(e).map((function(e) {
    var t = e[1];
    return {
      name: e[0],
      description: t.description,
      type: t.type,
      defaultValue: t.defaultValue,
      deprecationReason: t.deprecationReason,
      extensions: t.extensions && i(t.extensions),
      astNode: t.astNode
    };
  }));
}

function fieldsToFieldsConfig(e) {
  return n(e, (function(e) {
    return {
      description: e.description,
      type: e.type,
      args: argsToArgsConfig(e.args),
      resolve: e.resolve,
      subscribe: e.subscribe,
      deprecationReason: e.deprecationReason,
      extensions: e.extensions,
      astNode: e.astNode
    };
  }));
}

function argsToArgsConfig(e) {
  return s(e, (function(e) {
    return e.name;
  }), (function(e) {
    return {
      description: e.description,
      type: e.type,
      defaultValue: e.defaultValue,
      deprecationReason: e.deprecationReason,
      extensions: e.extensions,
      astNode: e.astNode
    };
  }));
}

function isRequiredArgument(e) {
  return isNonNullType(e.type) && void 0 === e.defaultValue;
}

var m = function GraphQLInterfaceType(e) {
  var t;
  this.name = e.name;
  this.description = e.description;
  this.resolveType = e.resolveType;
  this.extensions = e.extensions && i(e.extensions);
  this.astNode = e.astNode;
  this.extensionASTNodes = null !== (t = e.extensionASTNodes) && void 0 !== t ? t : [];
  this._fields = defineFieldMap.bind(void 0, e);
  this._interfaces = defineInterfaces.bind(void 0, e);
};

m.prototype.getFields = function getFields() {
  if ("function" == typeof this._fields) {
    this._fields = this._fields();
  }
  return this._fields;
};

m.prototype.getInterfaces = function getInterfaces() {
  if ("function" == typeof this._interfaces) {
    this._interfaces = this._interfaces();
  }
  return this._interfaces;
};

m.prototype.toConfig = function toConfig() {
  return {
    name: this.name,
    description: this.description,
    interfaces: this.getInterfaces(),
    fields: fieldsToFieldsConfig(this.getFields()),
    resolveType: this.resolveType,
    extensions: this.extensions,
    astNode: this.astNode,
    extensionASTNodes: this.extensionASTNodes
  };
};

m.prototype.toString = function toString() {
  return this.name;
};

m.prototype.toJSON = function toJSON() {
  return this.toString();
};

var N = function GraphQLUnionType(e) {
  var t;
  this.name = e.name;
  this.description = e.description;
  this.resolveType = e.resolveType;
  this.extensions = e.extensions && i(e.extensions);
  this.astNode = e.astNode;
  this.extensionASTNodes = null !== (t = e.extensionASTNodes) && void 0 !== t ? t : [];
  this._types = defineTypes.bind(void 0, e);
};

N.prototype.getTypes = function getTypes() {
  if ("function" == typeof this._types) {
    this._types = this._types();
  }
  return this._types;
};

N.prototype.toConfig = function toConfig() {
  return {
    name: this.name,
    description: this.description,
    types: this.getTypes(),
    resolveType: this.resolveType,
    extensions: this.extensions,
    astNode: this.astNode,
    extensionASTNodes: this.extensionASTNodes
  };
};

N.prototype.toString = function toString() {
  return this.name;
};

N.prototype.toJSON = function toJSON() {
  return this.toString();
};

function defineTypes(e) {
  return resolveReadonlyArrayThunk(e.types);
}

var g = function GraphQLEnumType(e) {
  var n;
  this.name = e.name;
  this.description = e.description;
  this.extensions = e.extensions && i(e.extensions);
  this.astNode = e.astNode;
  this.extensionASTNodes = null !== (n = e.extensionASTNodes) && void 0 !== n ? n : [];
  this._values = function defineEnumValues(e, t) {
    return Object.entries(t).map((function(e) {
      var t = e[0];
      var n = e[1];
      return {
        name: t,
        description: n.description,
        value: void 0 !== n.value ? n.value : t,
        deprecationReason: n.deprecationReason,
        extensions: n.extensions && i(n.extensions),
        astNode: n.astNode
      };
    }));
  }(0, e.values);
  this._valueLookup = new Map(this._values.map((function(e) {
    return [ e.value, e ];
  })));
  this._nameLookup = t(this._values, (function(e) {
    return e.name;
  }));
};

g.prototype.getValues = function getValues() {
  return this._values;
};

g.prototype.getValue = function getValue(e) {
  return this._nameLookup[e];
};

g.prototype.serialize = function serialize(t) {
  var n = this._valueLookup.get(t);
  if (void 0 === n) {
    throw new u('Enum "' + this.name + '" cannot represent value: ' + e(t));
  }
  return n.name;
};

g.prototype.parseValue = function parseValue(t) {
  if ("string" != typeof t) {
    var n = e(t);
    throw new u('Enum "' + this.name + '" cannot represent non-string value: ' + n + "." + didYouMeanEnumValue(this, n));
  }
  var i = this.getValue(t);
  if (null == i) {
    throw new u('Value "' + t + '" does not exist in "' + this.name + '" enum.' + didYouMeanEnumValue(this, t));
  }
  return i.value;
};

g.prototype.parseLiteral = function parseLiteral(e, t) {
  if (e.kind !== f.ENUM) {
    var n = c(e);
    throw new u('Enum "' + this.name + '" cannot represent non-enum value: ' + n + "." + didYouMeanEnumValue(this, n), e);
  }
  var i = this.getValue(e.value);
  if (null == i) {
    var s = c(e);
    throw new u('Value "' + s + '" does not exist in "' + this.name + '" enum.' + didYouMeanEnumValue(this, s), e);
  }
  return i.value;
};

g.prototype.toConfig = function toConfig() {
  var e = s(this.getValues(), (function(e) {
    return e.name;
  }), (function(e) {
    return {
      description: e.description,
      value: e.value,
      deprecationReason: e.deprecationReason,
      extensions: e.extensions,
      astNode: e.astNode
    };
  }));
  return {
    name: this.name,
    description: this.description,
    values: e,
    extensions: this.extensions,
    astNode: this.astNode,
    extensionASTNodes: this.extensionASTNodes
  };
};

g.prototype.toString = function toString() {
  return this.name;
};

g.prototype.toJSON = function toJSON() {
  return this.toString();
};

function didYouMeanEnumValue(e, t) {
  var n = e.getValues().map((function(e) {
    return e.name;
  }));
  var i = a(t, n);
  return r("the enum value", i);
}

var v = function GraphQLInputObjectType(e) {
  var t;
  this.name = e.name;
  this.description = e.description;
  this.extensions = e.extensions && i(e.extensions);
  this.astNode = e.astNode;
  this.extensionASTNodes = null !== (t = e.extensionASTNodes) && void 0 !== t ? t : [];
  this._fields = defineInputFieldMap.bind(void 0, e);
};

v.prototype.getFields = function getFields() {
  if ("function" == typeof this._fields) {
    this._fields = this._fields();
  }
  return this._fields;
};

v.prototype.toConfig = function toConfig() {
  var e = n(this.getFields(), (function(e) {
    return {
      description: e.description,
      type: e.type,
      defaultValue: e.defaultValue,
      extensions: e.extensions,
      astNode: e.astNode
    };
  }));
  return {
    name: this.name,
    description: this.description,
    fields: e,
    extensions: this.extensions,
    astNode: this.astNode,
    extensionASTNodes: this.extensionASTNodes
  };
};

v.prototype.toString = function toString() {
  return this.name;
};

v.prototype.toJSON = function toJSON() {
  return this.toString();
};

function defineInputFieldMap(e) {
  var t = resolveObjMapThunk(e.fields);
  return n(t, (function(e, t) {
    return {
      name: t,
      description: e.description,
      type: e.type,
      defaultValue: e.defaultValue,
      deprecationReason: e.deprecationReason,
      extensions: e.extensions && i(e.extensions),
      astNode: e.astNode
    };
  }));
}

function isRequiredInputField(e) {
  return isNonNullType(e.type) && void 0 === e.defaultValue;
}

export { g as GraphQLEnumType, v as GraphQLInputObjectType, m as GraphQLInterfaceType, y as GraphQLList, l as GraphQLNonNull, T as GraphQLObjectType, h as GraphQLScalarType, N as GraphQLUnionType, argsToArgsConfig, assertEnumType, assertInputObjectType, assertInputType, assertInterfaceType, assertLeafType, assertListType, assertNamedType, assertNonNullType, assertNullableType, assertObjectType, assertOutputType, assertScalarType, assertType, assertUnionType, assertWrappingType, defineArguments, getNamedType, getNullableType, isAbstractType, isCompositeType, isEnumType, isInputObjectType, isInputType, isInterfaceType, isLeafType, isListType, isNamedType, isNonNullType, isNullableType, isObjectType, isOutputType, isRequiredArgument, isRequiredInputField, isScalarType, isType, isUnionType, isWrappingType };
