import { inspect as e } from "../jsutils/inspect.mjs";

import { toObjMap as t } from "../jsutils/toObjMap.mjs";

import { instanceOf as i } from "../jsutils/instanceOf.mjs";

import { DirectiveLocation as s } from "../language/directiveLocation.mjs";

import { GraphQLBoolean as o, GraphQLString as r } from "./scalars.mjs";

import { defineArguments as n, argsToArgsConfig as a, GraphQLNonNull as c } from "./definition.mjs";

function isDirective(e) {
  return i(e, p);
}

function assertDirective(t) {
  if (!isDirective(t)) {
    throw new Error("Expected " + e(t) + " to be a GraphQL directive.");
  }
  return t;
}

var p = function GraphQLDirective(e) {
  var i, s;
  this.name = e.name;
  this.description = e.description;
  this.locations = e.locations;
  this.isRepeatable = null !== (i = e.isRepeatable) && void 0 !== i ? i : !1;
  this.extensions = e.extensions && t(e.extensions);
  this.astNode = e.astNode;
  var o = null !== (s = e.args) && void 0 !== s ? s : {};
  this.args = n(o);
};

p.prototype.toConfig = function toConfig() {
  return {
    name: this.name,
    description: this.description,
    locations: this.locations,
    args: a(this.args),
    isRepeatable: this.isRepeatable,
    extensions: this.extensions,
    astNode: this.astNode
  };
};

p.prototype.toString = function toString() {
  return "@" + this.name;
};

p.prototype.toJSON = function toJSON() {
  return this.toString();
};

var d = new p({
  name: "include",
  description: "Directs the executor to include this field or fragment only when the `if` argument is true.",
  locations: [ s.FIELD, s.FRAGMENT_SPREAD, s.INLINE_FRAGMENT ],
  args: {
    if: {
      type: new c(o),
      description: "Included when true."
    }
  }
});

var u = new p({
  name: "skip",
  description: "Directs the executor to skip this field or fragment when the `if` argument is true.",
  locations: [ s.FIELD, s.FRAGMENT_SPREAD, s.INLINE_FRAGMENT ],
  args: {
    if: {
      type: new c(o),
      description: "Skipped when true."
    }
  }
});

var h = "No longer supported";

var m = new p({
  name: "deprecated",
  description: "Marks an element of a GraphQL schema as no longer supported.",
  locations: [ s.FIELD_DEFINITION, s.ARGUMENT_DEFINITION, s.INPUT_FIELD_DEFINITION, s.ENUM_VALUE ],
  args: {
    reason: {
      type: r,
      description: "Explains why this element was deprecated, usually also including a suggestion for how to access supported similar data. Formatted using the Markdown syntax, as specified by [CommonMark](https://commonmark.org/).",
      defaultValue: "No longer supported"
    }
  }
});

var l = new p({
  name: "specifiedBy",
  description: "Exposes a URL that specifies the behaviour of this scalar.",
  locations: [ s.SCALAR ],
  args: {
    url: {
      type: new c(r),
      description: "The URL that specifies the behaviour of this scalar."
    }
  }
});

var f = [ d, u, m, l ];

function isSpecifiedDirective(e) {
  return f.some((function(t) {
    return t.name === e.name;
  }));
}

export { h as DEFAULT_DEPRECATION_REASON, m as GraphQLDeprecatedDirective, p as GraphQLDirective, d as GraphQLIncludeDirective, u as GraphQLSkipDirective, l as GraphQLSpecifiedByDirective, assertDirective, isDirective, isSpecifiedDirective, f as specifiedDirectives };
