"use strict";

Object.defineProperty(exports, "__esModule", {
  value: !0
});

var e = require("../jsutils/invariant.js");

var n = require("../language/printer.js");

var t = require("../language/directiveLocation.js");

var i = require("../utilities/astFromValue.js");

var r = require("./scalars.js");

var a = require("./definition.js");

var o = new a.GraphQLObjectType({
  name: "__Schema",
  description: "A GraphQL Schema defines the capabilities of a GraphQL server. It exposes all available types and directives on the server, as well as the entry points for query, mutation, and subscription operations.",
  fields: function() {
    return {
      description: {
        type: r.GraphQLString,
        resolve: function(e) {
          return e.description;
        }
      },
      types: {
        description: "A list of all types supported by this server.",
        type: new a.GraphQLNonNull(new a.GraphQLList(new a.GraphQLNonNull(c))),
        resolve: function resolve(e) {
          return Object.values(e.getTypeMap());
        }
      },
      queryType: {
        description: "The type that query operations will be rooted at.",
        type: new a.GraphQLNonNull(c),
        resolve: function(e) {
          return e.getQueryType();
        }
      },
      mutationType: {
        description: "If this server supports mutation, the type that mutation operations will be rooted at.",
        type: c,
        resolve: function(e) {
          return e.getMutationType();
        }
      },
      subscriptionType: {
        description: "If this server support subscription, the type that subscription operations will be rooted at.",
        type: c,
        resolve: function(e) {
          return e.getSubscriptionType();
        }
      },
      directives: {
        description: "A list of all directives supported by this server.",
        type: new a.GraphQLNonNull(new a.GraphQLList(new a.GraphQLNonNull(s))),
        resolve: function(e) {
          return e.getDirectives();
        }
      }
    };
  }
});

var s = new a.GraphQLObjectType({
  name: "__Directive",
  description: "A Directive provides a way to describe alternate runtime execution and type validation behavior in a GraphQL document.\n\nIn some cases, you need to provide options to alter GraphQL's execution behavior in ways field arguments will not suffice, such as conditionally including or skipping a field. Directives provide this by describing additional information to the executor.",
  fields: function() {
    return {
      name: {
        type: new a.GraphQLNonNull(r.GraphQLString),
        resolve: function(e) {
          return e.name;
        }
      },
      description: {
        type: r.GraphQLString,
        resolve: function(e) {
          return e.description;
        }
      },
      isRepeatable: {
        type: new a.GraphQLNonNull(r.GraphQLBoolean),
        resolve: function(e) {
          return e.isRepeatable;
        }
      },
      locations: {
        type: new a.GraphQLNonNull(new a.GraphQLList(new a.GraphQLNonNull(p))),
        resolve: function(e) {
          return e.locations;
        }
      },
      args: {
        type: new a.GraphQLNonNull(new a.GraphQLList(new a.GraphQLNonNull(u))),
        resolve: function(e) {
          return e.args;
        }
      }
    };
  }
});

var p = new a.GraphQLEnumType({
  name: "__DirectiveLocation",
  description: "A Directive can be adjacent to many parts of the GraphQL language, a __DirectiveLocation describes one such possible adjacencies.",
  values: {
    QUERY: {
      value: t.DirectiveLocation.QUERY,
      description: "Location adjacent to a query operation."
    },
    MUTATION: {
      value: t.DirectiveLocation.MUTATION,
      description: "Location adjacent to a mutation operation."
    },
    SUBSCRIPTION: {
      value: t.DirectiveLocation.SUBSCRIPTION,
      description: "Location adjacent to a subscription operation."
    },
    FIELD: {
      value: t.DirectiveLocation.FIELD,
      description: "Location adjacent to a field."
    },
    FRAGMENT_DEFINITION: {
      value: t.DirectiveLocation.FRAGMENT_DEFINITION,
      description: "Location adjacent to a fragment definition."
    },
    FRAGMENT_SPREAD: {
      value: t.DirectiveLocation.FRAGMENT_SPREAD,
      description: "Location adjacent to a fragment spread."
    },
    INLINE_FRAGMENT: {
      value: t.DirectiveLocation.INLINE_FRAGMENT,
      description: "Location adjacent to an inline fragment."
    },
    VARIABLE_DEFINITION: {
      value: t.DirectiveLocation.VARIABLE_DEFINITION,
      description: "Location adjacent to a variable definition."
    },
    SCHEMA: {
      value: t.DirectiveLocation.SCHEMA,
      description: "Location adjacent to a schema definition."
    },
    SCALAR: {
      value: t.DirectiveLocation.SCALAR,
      description: "Location adjacent to a scalar definition."
    },
    OBJECT: {
      value: t.DirectiveLocation.OBJECT,
      description: "Location adjacent to an object type definition."
    },
    FIELD_DEFINITION: {
      value: t.DirectiveLocation.FIELD_DEFINITION,
      description: "Location adjacent to a field definition."
    },
    ARGUMENT_DEFINITION: {
      value: t.DirectiveLocation.ARGUMENT_DEFINITION,
      description: "Location adjacent to an argument definition."
    },
    INTERFACE: {
      value: t.DirectiveLocation.INTERFACE,
      description: "Location adjacent to an interface definition."
    },
    UNION: {
      value: t.DirectiveLocation.UNION,
      description: "Location adjacent to a union definition."
    },
    ENUM: {
      value: t.DirectiveLocation.ENUM,
      description: "Location adjacent to an enum definition."
    },
    ENUM_VALUE: {
      value: t.DirectiveLocation.ENUM_VALUE,
      description: "Location adjacent to an enum value definition."
    },
    INPUT_OBJECT: {
      value: t.DirectiveLocation.INPUT_OBJECT,
      description: "Location adjacent to an input object type definition."
    },
    INPUT_FIELD_DEFINITION: {
      value: t.DirectiveLocation.INPUT_FIELD_DEFINITION,
      description: "Location adjacent to an input object field definition."
    }
  }
});

var c = new a.GraphQLObjectType({
  name: "__Type",
  description: "The fundamental unit of any GraphQL Schema is the type. There are many kinds of types in GraphQL as represented by the `__TypeKind` enum.\n\nDepending on the kind of a type, certain fields describe information about that type. Scalar types provide no information beyond a name, description and optional `specifiedByURL`, while Enum types provide their values. Object and Interface types provide the fields they describe. Abstract types, Union and Interface, provide the Object types possible at runtime. List and NonNull types compose other types.",
  fields: function() {
    return {
      kind: {
        type: new a.GraphQLNonNull(f),
        resolve: function resolve(n) {
          if (a.isScalarType(n)) {
            return v.SCALAR;
          }
          if (a.isObjectType(n)) {
            return v.OBJECT;
          }
          if (a.isInterfaceType(n)) {
            return v.INTERFACE;
          }
          if (a.isUnionType(n)) {
            return v.UNION;
          }
          if (a.isEnumType(n)) {
            return v.ENUM;
          }
          if (a.isInputObjectType(n)) {
            return v.INPUT_OBJECT;
          }
          if (a.isListType(n)) {
            return v.LIST;
          }
          if (a.isNonNullType(n)) {
            return v.NON_NULL;
          }
          e.invariant(!1);
        }
      },
      name: {
        type: r.GraphQLString,
        resolve: function(e) {
          return "name" in e ? e.name : void 0;
        }
      },
      description: {
        type: r.GraphQLString,
        resolve: function(e) {
          return "description" in e ? e.description : void 0;
        }
      },
      specifiedByURL: {
        type: r.GraphQLString,
        resolve: function(e) {
          return "specifiedByURL" in e ? e.specifiedByURL : void 0;
        }
      },
      fields: {
        type: new a.GraphQLList(new a.GraphQLNonNull(l)),
        args: {
          includeDeprecated: {
            type: r.GraphQLBoolean,
            defaultValue: !1
          }
        },
        resolve: function resolve(e, n) {
          var t = n.includeDeprecated;
          if (a.isObjectType(e) || a.isInterfaceType(e)) {
            var i = Object.values(e.getFields());
            return t ? i : i.filter((function(e) {
              return null == e.deprecationReason;
            }));
          }
        }
      },
      interfaces: {
        type: new a.GraphQLList(new a.GraphQLNonNull(c)),
        resolve: function resolve(e) {
          if (a.isObjectType(e) || a.isInterfaceType(e)) {
            return e.getInterfaces();
          }
        }
      },
      possibleTypes: {
        type: new a.GraphQLList(new a.GraphQLNonNull(c)),
        resolve: function resolve(e, n, t, i) {
          var r = i.schema;
          if (a.isAbstractType(e)) {
            return r.getPossibleTypes(e);
          }
        }
      },
      enumValues: {
        type: new a.GraphQLList(new a.GraphQLNonNull(d)),
        args: {
          includeDeprecated: {
            type: r.GraphQLBoolean,
            defaultValue: !1
          }
        },
        resolve: function resolve(e, n) {
          var t = n.includeDeprecated;
          if (a.isEnumType(e)) {
            var i = e.getValues();
            return t ? i : i.filter((function(e) {
              return null == e.deprecationReason;
            }));
          }
        }
      },
      inputFields: {
        type: new a.GraphQLList(new a.GraphQLNonNull(u)),
        args: {
          includeDeprecated: {
            type: r.GraphQLBoolean,
            defaultValue: !1
          }
        },
        resolve: function resolve(e, n) {
          var t = n.includeDeprecated;
          if (a.isInputObjectType(e)) {
            var i = Object.values(e.getFields());
            return t ? i : i.filter((function(e) {
              return null == e.deprecationReason;
            }));
          }
        }
      },
      ofType: {
        type: c,
        resolve: function(e) {
          return "ofType" in e ? e.ofType : void 0;
        }
      }
    };
  }
});

var l = new a.GraphQLObjectType({
  name: "__Field",
  description: "Object and Interface types are described by a list of Fields, each of which has a name, potentially a list of arguments, and a return type.",
  fields: function() {
    return {
      name: {
        type: new a.GraphQLNonNull(r.GraphQLString),
        resolve: function(e) {
          return e.name;
        }
      },
      description: {
        type: r.GraphQLString,
        resolve: function(e) {
          return e.description;
        }
      },
      args: {
        type: new a.GraphQLNonNull(new a.GraphQLList(new a.GraphQLNonNull(u))),
        args: {
          includeDeprecated: {
            type: r.GraphQLBoolean,
            defaultValue: !1
          }
        },
        resolve: function resolve(e, n) {
          return n.includeDeprecated ? e.args : e.args.filter((function(e) {
            return null == e.deprecationReason;
          }));
        }
      },
      type: {
        type: new a.GraphQLNonNull(c),
        resolve: function(e) {
          return e.type;
        }
      },
      isDeprecated: {
        type: new a.GraphQLNonNull(r.GraphQLBoolean),
        resolve: function(e) {
          return null != e.deprecationReason;
        }
      },
      deprecationReason: {
        type: r.GraphQLString,
        resolve: function(e) {
          return e.deprecationReason;
        }
      }
    };
  }
});

var u = new a.GraphQLObjectType({
  name: "__InputValue",
  description: "Arguments provided to Fields or Directives and the input fields of an InputObject are represented as Input Values which describe their type and optionally a default value.",
  fields: function() {
    return {
      name: {
        type: new a.GraphQLNonNull(r.GraphQLString),
        resolve: function(e) {
          return e.name;
        }
      },
      description: {
        type: r.GraphQLString,
        resolve: function(e) {
          return e.description;
        }
      },
      type: {
        type: new a.GraphQLNonNull(c),
        resolve: function(e) {
          return e.type;
        }
      },
      defaultValue: {
        type: r.GraphQLString,
        description: "A GraphQL-formatted string representing the default value for this input value.",
        resolve: function resolve(e) {
          var t = i.astFromValue(e.defaultValue, e.type);
          return t ? n.print(t) : null;
        }
      },
      isDeprecated: {
        type: new a.GraphQLNonNull(r.GraphQLBoolean),
        resolve: function(e) {
          return null != e.deprecationReason;
        }
      },
      deprecationReason: {
        type: r.GraphQLString,
        resolve: function(e) {
          return e.deprecationReason;
        }
      }
    };
  }
});

var d = new a.GraphQLObjectType({
  name: "__EnumValue",
  description: "One possible value for a given Enum. Enum values are unique values, not a placeholder for a string or numeric value. However an Enum value is returned in a JSON response as a string.",
  fields: function() {
    return {
      name: {
        type: new a.GraphQLNonNull(r.GraphQLString),
        resolve: function(e) {
          return e.name;
        }
      },
      description: {
        type: r.GraphQLString,
        resolve: function(e) {
          return e.description;
        }
      },
      isDeprecated: {
        type: new a.GraphQLNonNull(r.GraphQLBoolean),
        resolve: function(e) {
          return null != e.deprecationReason;
        }
      },
      deprecationReason: {
        type: r.GraphQLString,
        resolve: function(e) {
          return e.deprecationReason;
        }
      }
    };
  }
});

var v = {
  SCALAR: "SCALAR",
  OBJECT: "OBJECT",
  INTERFACE: "INTERFACE",
  UNION: "UNION",
  ENUM: "ENUM",
  INPUT_OBJECT: "INPUT_OBJECT",
  LIST: "LIST",
  NON_NULL: "NON_NULL"
};

var f = new a.GraphQLEnumType({
  name: "__TypeKind",
  description: "An enum describing what kind of type a given `__Type` is.",
  values: {
    SCALAR: {
      value: v.SCALAR,
      description: "Indicates this type is a scalar."
    },
    OBJECT: {
      value: v.OBJECT,
      description: "Indicates this type is an object. `fields` and `interfaces` are valid fields."
    },
    INTERFACE: {
      value: v.INTERFACE,
      description: "Indicates this type is an interface. `fields`, `interfaces`, and `possibleTypes` are valid fields."
    },
    UNION: {
      value: v.UNION,
      description: "Indicates this type is a union. `possibleTypes` is a valid field."
    },
    ENUM: {
      value: v.ENUM,
      description: "Indicates this type is an enum. `enumValues` is a valid field."
    },
    INPUT_OBJECT: {
      value: v.INPUT_OBJECT,
      description: "Indicates this type is an input object. `inputFields` is a valid field."
    },
    LIST: {
      value: v.LIST,
      description: "Indicates this type is a list. `ofType` is a valid field."
    },
    NON_NULL: {
      value: v.NON_NULL,
      description: "Indicates this type is a non-null. `ofType` is a valid field."
    }
  }
});

var L = {
  name: "__schema",
  type: new a.GraphQLNonNull(o),
  description: "Access the current type schema of this server.",
  args: [],
  resolve: function(e, n, t, i) {
    return i.schema;
  },
  deprecationReason: void 0,
  extensions: void 0,
  astNode: void 0
};

var y = {
  name: "__type",
  type: c,
  description: "Request the type information of a single type.",
  args: [ {
    name: "name",
    description: void 0,
    type: new a.GraphQLNonNull(r.GraphQLString),
    defaultValue: void 0,
    deprecationReason: void 0,
    extensions: void 0,
    astNode: void 0
  } ],
  resolve: function(e, n, t, i) {
    return i.schema.getType(n.name);
  },
  deprecationReason: void 0,
  extensions: void 0,
  astNode: void 0
};

var N = {
  name: "__typename",
  type: new a.GraphQLNonNull(r.GraphQLString),
  description: "The name of the current Object type at runtime.",
  args: [],
  resolve: function(e, n, t, i) {
    return i.parentType.name;
  },
  deprecationReason: void 0,
  extensions: void 0,
  astNode: void 0
};

var h = [ o, s, p, c, l, u, d, f ];

exports.SchemaMetaFieldDef = L;

exports.TypeKind = v;

exports.TypeMetaFieldDef = y;

exports.TypeNameMetaFieldDef = N;

exports.__Directive = s;

exports.__DirectiveLocation = p;

exports.__EnumValue = d;

exports.__Field = l;

exports.__InputValue = u;

exports.__Schema = o;

exports.__Type = c;

exports.__TypeKind = f;

exports.introspectionTypes = h;

exports.isIntrospectionType = function isIntrospectionType(e) {
  return h.some((function(n) {
    return e.name === n.name;
  }));
};
