"use strict";

Object.defineProperty(exports, "__esModule", {
  value: !0
});

var e = require("../jsutils/inspect.js");

var r = require("../jsutils/isObjectLike.js");

var n = require("../language/kinds.js");

var t = require("../language/printer.js");

var a = require("../error/GraphQLError.js");

var i = require("./definition.js");

var o = new i.GraphQLScalarType({
  name: "Int",
  description: "The `Int` scalar type represents non-fractional signed whole numeric values. Int can represent values between -(2^31) and 2^31 - 1.",
  serialize: function serializeInt(r) {
    var n = serializeObject(r);
    if ("boolean" == typeof n) {
      return n ? 1 : 0;
    }
    var t = n;
    if ("string" == typeof n && "" !== n) {
      t = Number(n);
    }
    if ("number" != typeof t || !Number.isInteger(t)) {
      throw new a.GraphQLError("Int cannot represent non-integer value: " + e.inspect(n));
    }
    if (t > 2147483647 || t < -2147483648) {
      throw new a.GraphQLError("Int cannot represent non 32-bit signed integer value: " + e.inspect(n));
    }
    return t;
  },
  parseValue: function coerceInt(r) {
    if ("number" != typeof r || !Number.isInteger(r)) {
      throw new a.GraphQLError("Int cannot represent non-integer value: " + e.inspect(r));
    }
    if (r > 2147483647 || r < -2147483648) {
      throw new a.GraphQLError("Int cannot represent non 32-bit signed integer value: " + r);
    }
    return r;
  },
  parseLiteral: function parseLiteral(e) {
    if (e.kind !== n.Kind.INT) {
      throw new a.GraphQLError("Int cannot represent non-integer value: " + t.print(e), e);
    }
    var r = parseInt(e.value, 10);
    if (r > 2147483647 || r < -2147483648) {
      throw new a.GraphQLError("Int cannot represent non 32-bit signed integer value: " + e.value, e);
    }
    return r;
  }
});

var s = new i.GraphQLScalarType({
  name: "Float",
  description: "The `Float` scalar type represents signed double-precision fractional values as specified by [IEEE 754](https://en.wikipedia.org/wiki/IEEE_floating_point).",
  serialize: function serializeFloat(r) {
    var n = serializeObject(r);
    if ("boolean" == typeof n) {
      return n ? 1 : 0;
    }
    var t = n;
    if ("string" == typeof n && "" !== n) {
      t = Number(n);
    }
    if ("number" != typeof t || !Number.isFinite(t)) {
      throw new a.GraphQLError("Float cannot represent non numeric value: " + e.inspect(n));
    }
    return t;
  },
  parseValue: function coerceFloat(r) {
    if ("number" != typeof r || !Number.isFinite(r)) {
      throw new a.GraphQLError("Float cannot represent non numeric value: " + e.inspect(r));
    }
    return r;
  },
  parseLiteral: function parseLiteral(e) {
    if (e.kind !== n.Kind.FLOAT && e.kind !== n.Kind.INT) {
      throw new a.GraphQLError("Float cannot represent non numeric value: " + t.print(e), e);
    }
    return parseFloat(e.value);
  }
});

function serializeObject(e) {
  if (r.isObjectLike(e)) {
    if ("function" == typeof e.valueOf) {
      var n = e.valueOf();
      if (!r.isObjectLike(n)) {
        return n;
      }
    }
    if ("function" == typeof e.toJSON) {
      return e.toJSON();
    }
  }
  return e;
}

var p = new i.GraphQLScalarType({
  name: "String",
  description: "The `String` scalar type represents textual data, represented as UTF-8 character sequences. The String type is most often used by GraphQL to represent free-form human-readable text.",
  serialize: function serializeString(r) {
    var n = serializeObject(r);
    if ("string" == typeof n) {
      return n;
    }
    if ("boolean" == typeof n) {
      return n ? "true" : "false";
    }
    if ("number" == typeof n && Number.isFinite(n)) {
      return n.toString();
    }
    throw new a.GraphQLError("String cannot represent value: " + e.inspect(r));
  },
  parseValue: function coerceString(r) {
    if ("string" != typeof r) {
      throw new a.GraphQLError("String cannot represent a non string value: " + e.inspect(r));
    }
    return r;
  },
  parseLiteral: function parseLiteral(e) {
    if (e.kind !== n.Kind.STRING) {
      throw new a.GraphQLError("String cannot represent a non string value: " + t.print(e), e);
    }
    return e.value;
  }
});

var u = new i.GraphQLScalarType({
  name: "Boolean",
  description: "The `Boolean` scalar type represents `true` or `false`.",
  serialize: function serializeBoolean(r) {
    var n = serializeObject(r);
    if ("boolean" == typeof n) {
      return n;
    }
    if (Number.isFinite(n)) {
      return 0 !== n;
    }
    throw new a.GraphQLError("Boolean cannot represent a non boolean value: " + e.inspect(n));
  },
  parseValue: function coerceBoolean(r) {
    if ("boolean" != typeof r) {
      throw new a.GraphQLError("Boolean cannot represent a non boolean value: " + e.inspect(r));
    }
    return r;
  },
  parseLiteral: function parseLiteral(e) {
    if (e.kind !== n.Kind.BOOLEAN) {
      throw new a.GraphQLError("Boolean cannot represent a non boolean value: " + t.print(e), e);
    }
    return e.value;
  }
});

var l = new i.GraphQLScalarType({
  name: "ID",
  description: 'The `ID` scalar type represents a unique identifier, often used to refetch an object or as key for a cache. The ID type appears in a JSON response as a String; however, it is not intended to be human-readable. When expected as an input type, any string (such as `"4"`) or integer (such as `4`) input value will be accepted as an ID.',
  serialize: function serializeID(r) {
    var n = serializeObject(r);
    if ("string" == typeof n) {
      return n;
    }
    if (Number.isInteger(n)) {
      return String(n);
    }
    throw new a.GraphQLError("ID cannot represent value: " + e.inspect(r));
  },
  parseValue: function coerceID(r) {
    if ("string" == typeof r) {
      return r;
    }
    if ("number" == typeof r && Number.isInteger(r)) {
      return r.toString();
    }
    throw new a.GraphQLError("ID cannot represent value: " + e.inspect(r));
  },
  parseLiteral: function parseLiteral(e) {
    if (e.kind !== n.Kind.STRING && e.kind !== n.Kind.INT) {
      throw new a.GraphQLError("ID cannot represent a non-string and non-integer value: " + t.print(e), e);
    }
    return e.value;
  }
});

var c = [ p, o, s, u, l ];

exports.GraphQLBoolean = u;

exports.GraphQLFloat = s;

exports.GraphQLID = l;

exports.GraphQLInt = o;

exports.GraphQLString = p;

exports.isSpecifiedScalarType = function isSpecifiedScalarType(e) {
  return c.some((function(r) {
    return e.name === r.name;
  }));
};

exports.specifiedScalarTypes = c;
