import { inspect as e } from "../jsutils/inspect.mjs";

import { isObjectLike as n } from "../jsutils/isObjectLike.mjs";

import { Kind as r } from "../language/kinds.mjs";

import { print as t } from "../language/printer.mjs";

import { GraphQLError as i } from "../error/GraphQLError.mjs";

import { GraphQLScalarType as a } from "./definition.mjs";

var o = new a({
  name: "Int",
  description: "The `Int` scalar type represents non-fractional signed whole numeric values. Int can represent values between -(2^31) and 2^31 - 1.",
  serialize: function serializeInt(n) {
    var r = serializeObject(n);
    if ("boolean" == typeof r) {
      return r ? 1 : 0;
    }
    var t = r;
    if ("string" == typeof r && "" !== r) {
      t = Number(r);
    }
    if ("number" != typeof t || !Number.isInteger(t)) {
      throw new i("Int cannot represent non-integer value: " + e(r));
    }
    if (t > 2147483647 || t < -2147483648) {
      throw new i("Int cannot represent non 32-bit signed integer value: " + e(r));
    }
    return t;
  },
  parseValue: function coerceInt(n) {
    if ("number" != typeof n || !Number.isInteger(n)) {
      throw new i("Int cannot represent non-integer value: " + e(n));
    }
    if (n > 2147483647 || n < -2147483648) {
      throw new i("Int cannot represent non 32-bit signed integer value: " + n);
    }
    return n;
  },
  parseLiteral: function parseLiteral(e) {
    if (e.kind !== r.INT) {
      throw new i("Int cannot represent non-integer value: " + t(e), e);
    }
    var n = parseInt(e.value, 10);
    if (n > 2147483647 || n < -2147483648) {
      throw new i("Int cannot represent non 32-bit signed integer value: " + e.value, e);
    }
    return n;
  }
});

var s = new a({
  name: "Float",
  description: "The `Float` scalar type represents signed double-precision fractional values as specified by [IEEE 754](https://en.wikipedia.org/wiki/IEEE_floating_point).",
  serialize: function serializeFloat(n) {
    var r = serializeObject(n);
    if ("boolean" == typeof r) {
      return r ? 1 : 0;
    }
    var t = r;
    if ("string" == typeof r && "" !== r) {
      t = Number(r);
    }
    if ("number" != typeof t || !Number.isFinite(t)) {
      throw new i("Float cannot represent non numeric value: " + e(r));
    }
    return t;
  },
  parseValue: function coerceFloat(n) {
    if ("number" != typeof n || !Number.isFinite(n)) {
      throw new i("Float cannot represent non numeric value: " + e(n));
    }
    return n;
  },
  parseLiteral: function parseLiteral(e) {
    if (e.kind !== r.FLOAT && e.kind !== r.INT) {
      throw new i("Float cannot represent non numeric value: " + t(e), e);
    }
    return parseFloat(e.value);
  }
});

function serializeObject(e) {
  if (n(e)) {
    if ("function" == typeof e.valueOf) {
      var r = e.valueOf();
      if (!n(r)) {
        return r;
      }
    }
    if ("function" == typeof e.toJSON) {
      return e.toJSON();
    }
  }
  return e;
}

var u = new a({
  name: "String",
  description: "The `String` scalar type represents textual data, represented as UTF-8 character sequences. The String type is most often used by GraphQL to represent free-form human-readable text.",
  serialize: function serializeString(n) {
    var r = serializeObject(n);
    if ("string" == typeof r) {
      return r;
    }
    if ("boolean" == typeof r) {
      return r ? "true" : "false";
    }
    if ("number" == typeof r && Number.isFinite(r)) {
      return r.toString();
    }
    throw new i("String cannot represent value: " + e(n));
  },
  parseValue: function coerceString(n) {
    if ("string" != typeof n) {
      throw new i("String cannot represent a non string value: " + e(n));
    }
    return n;
  },
  parseLiteral: function parseLiteral(e) {
    if (e.kind !== r.STRING) {
      throw new i("String cannot represent a non string value: " + t(e), e);
    }
    return e.value;
  }
});

var l = new a({
  name: "Boolean",
  description: "The `Boolean` scalar type represents `true` or `false`.",
  serialize: function serializeBoolean(n) {
    var r = serializeObject(n);
    if ("boolean" == typeof r) {
      return r;
    }
    if (Number.isFinite(r)) {
      return 0 !== r;
    }
    throw new i("Boolean cannot represent a non boolean value: " + e(r));
  },
  parseValue: function coerceBoolean(n) {
    if ("boolean" != typeof n) {
      throw new i("Boolean cannot represent a non boolean value: " + e(n));
    }
    return n;
  },
  parseLiteral: function parseLiteral(e) {
    if (e.kind !== r.BOOLEAN) {
      throw new i("Boolean cannot represent a non boolean value: " + t(e), e);
    }
    return e.value;
  }
});

var p = new a({
  name: "ID",
  description: 'The `ID` scalar type represents a unique identifier, often used to refetch an object or as key for a cache. The ID type appears in a JSON response as a String; however, it is not intended to be human-readable. When expected as an input type, any string (such as `"4"`) or integer (such as `4`) input value will be accepted as an ID.',
  serialize: function serializeID(n) {
    var r = serializeObject(n);
    if ("string" == typeof r) {
      return r;
    }
    if (Number.isInteger(r)) {
      return String(r);
    }
    throw new i("ID cannot represent value: " + e(n));
  },
  parseValue: function coerceID(n) {
    if ("string" == typeof n) {
      return n;
    }
    if ("number" == typeof n && Number.isInteger(n)) {
      return n.toString();
    }
    throw new i("ID cannot represent value: " + e(n));
  },
  parseLiteral: function parseLiteral(e) {
    if (e.kind !== r.STRING && e.kind !== r.INT) {
      throw new i("ID cannot represent a non-string and non-integer value: " + t(e), e);
    }
    return e.value;
  }
});

var f = [ u, o, s, l, p ];

function isSpecifiedScalarType(e) {
  return f.some((function(n) {
    return e.name === n.name;
  }));
}

export { l as GraphQLBoolean, s as GraphQLFloat, p as GraphQLID, o as GraphQLInt, u as GraphQLString, isSpecifiedScalarType, f as specifiedScalarTypes };
