"use strict";

Object.defineProperty(exports, "__esModule", {
  value: !0
});

var e = require("../jsutils/inspect.js");

var t = require("../jsutils/toObjMap.js");

var i = require("../jsutils/instanceOf.js");

var s = require("./introspection.js");

var n = require("./directives.js");

var r = require("./definition.js");

var o = function GraphQLSchema(e) {
  var i, o;
  this.__validationErrors = !0 === e.assumeValid ? [] : void 0;
  this.description = e.description;
  this.extensions = e.extensions && t.toObjMap(e.extensions);
  this.astNode = e.astNode;
  this.extensionASTNodes = null !== (i = e.extensionASTNodes) && void 0 !== i ? i : [];
  this._queryType = e.query;
  this._mutationType = e.mutation;
  this._subscriptionType = e.subscription;
  this._directives = null !== (o = e.directives) && void 0 !== o ? o : n.specifiedDirectives;
  var p = new Set(e.types);
  if (null != e.types) {
    for (var a = 0, c = e.types; a < c.length; a += 1) {
      var u = c[a];
      p.delete(u);
      collectReferencedTypes(u, p);
    }
  }
  if (null != this._queryType) {
    collectReferencedTypes(this._queryType, p);
  }
  if (null != this._mutationType) {
    collectReferencedTypes(this._mutationType, p);
  }
  if (null != this._subscriptionType) {
    collectReferencedTypes(this._subscriptionType, p);
  }
  for (var l = 0, y = this._directives; l < y.length; l += 1) {
    var f = y[l];
    if (n.isDirective(f)) {
      for (var h = 0, v = f.args; h < v.length; h += 1) {
        collectReferencedTypes(v[h].type, p);
      }
    }
  }
  collectReferencedTypes(s.__Schema, p);
  this._typeMap = Object.create(null);
  this._subTypeMap = Object.create(null);
  this._implementationsMap = Object.create(null);
  for (var T = 0, d = p; T < d.length; T += 1) {
    var g = d[T];
    if (null == g) {
      continue;
    }
    var m = g.name;
    if (void 0 !== this._typeMap[m]) {
      throw new Error('Schema must contain uniquely named types but contains multiple types named "' + m + '".');
    }
    this._typeMap[m] = g;
    if (r.isInterfaceType(g)) {
      for (var b = 0, _ = g.getInterfaces(); b < _.length; b += 1) {
        var j = _[b];
        if (r.isInterfaceType(j)) {
          var M = this._implementationsMap[j.name];
          if (void 0 === M) {
            M = this._implementationsMap[j.name] = {
              objects: [],
              interfaces: []
            };
          }
          M.interfaces.push(g);
        }
      }
    } else if (r.isObjectType(g)) {
      for (var S = 0, O = g.getInterfaces(); S < O.length; S += 1) {
        var q = O[S];
        if (r.isInterfaceType(q)) {
          var x = this._implementationsMap[q.name];
          if (void 0 === x) {
            x = this._implementationsMap[q.name] = {
              objects: [],
              interfaces: []
            };
          }
          x.objects.push(g);
        }
      }
    }
  }
};

o.prototype.getQueryType = function getQueryType() {
  return this._queryType;
};

o.prototype.getMutationType = function getMutationType() {
  return this._mutationType;
};

o.prototype.getSubscriptionType = function getSubscriptionType() {
  return this._subscriptionType;
};

o.prototype.getTypeMap = function getTypeMap() {
  return this._typeMap;
};

o.prototype.getType = function getType(e) {
  return this.getTypeMap()[e];
};

o.prototype.getPossibleTypes = function getPossibleTypes(e) {
  return r.isUnionType(e) ? e.getTypes() : this.getImplementations(e).objects;
};

o.prototype.getImplementations = function getImplementations(e) {
  var t = this._implementationsMap[e.name];
  return null != t ? t : {
    objects: [],
    interfaces: []
  };
};

o.prototype.isSubType = function isSubType(e, t) {
  var i = this._subTypeMap[e.name];
  if (void 0 === i) {
    i = Object.create(null);
    if (r.isUnionType(e)) {
      for (var s = 0, n = e.getTypes(); s < n.length; s += 1) {
        i[n[s].name] = !0;
      }
    } else {
      var o = this.getImplementations(e);
      for (var p = 0, a = o.objects; p < a.length; p += 1) {
        i[a[p].name] = !0;
      }
      for (var c = 0, u = o.interfaces; c < u.length; c += 1) {
        i[u[c].name] = !0;
      }
    }
    this._subTypeMap[e.name] = i;
  }
  return void 0 !== i[t.name];
};

o.prototype.getDirectives = function getDirectives() {
  return this._directives;
};

o.prototype.getDirective = function getDirective(e) {
  return this.getDirectives().find((function(t) {
    return t.name === e;
  }));
};

o.prototype.toConfig = function toConfig() {
  return {
    description: this.description,
    query: this.getQueryType(),
    mutation: this.getMutationType(),
    subscription: this.getSubscriptionType(),
    types: Object.values(this.getTypeMap()),
    directives: this.getDirectives(),
    extensions: this.extensions,
    astNode: this.astNode,
    extensionASTNodes: this.extensionASTNodes,
    assumeValid: void 0 !== this.__validationErrors
  };
};

function collectReferencedTypes(e, t) {
  var i = r.getNamedType(e);
  if (!t.has(i)) {
    t.add(i);
    if (r.isUnionType(i)) {
      for (var s = 0, n = i.getTypes(); s < n.length; s += 1) {
        collectReferencedTypes(n[s], t);
      }
    } else if (r.isObjectType(i) || r.isInterfaceType(i)) {
      for (var o = 0, p = i.getInterfaces(); o < p.length; o += 1) {
        collectReferencedTypes(p[o], t);
      }
      for (var a = 0, c = Object.values(i.getFields()); a < c.length; a += 1) {
        var u = c[a];
        collectReferencedTypes(u.type, t);
        for (var l = 0, y = u.args; l < y.length; l += 1) {
          collectReferencedTypes(y[l].type, t);
        }
      }
    } else if (r.isInputObjectType(i)) {
      for (var f = 0, h = Object.values(i.getFields()); f < h.length; f += 1) {
        collectReferencedTypes(h[f].type, t);
      }
    }
  }
  return t;
}

exports.GraphQLSchema = o;

exports.assertSchema = function assertSchema(t) {
  if (!function isSchema(e) {
    return i.instanceOf(e, o);
  }(t)) {
    throw new Error("Expected " + e.inspect(t) + " to be a GraphQL schema.");
  }
  return t;
};
