import { inspect as e } from "../jsutils/inspect.mjs";

import { toObjMap as t } from "../jsutils/toObjMap.mjs";

import { instanceOf as i } from "../jsutils/instanceOf.mjs";

import { __Schema as s } from "./introspection.mjs";

import { specifiedDirectives as n, isDirective as r } from "./directives.mjs";

import { isInterfaceType as o, isObjectType as p, isUnionType as a, getNamedType as c, isInputObjectType as l } from "./definition.mjs";

function assertSchema(t) {
  if (!function isSchema(e) {
    return i(e, u);
  }(t)) {
    throw new Error("Expected " + e(t) + " to be a GraphQL schema.");
  }
  return t;
}

var u = function GraphQLSchema(e) {
  var i, a;
  this.__validationErrors = !0 === e.assumeValid ? [] : void 0;
  this.description = e.description;
  this.extensions = e.extensions && t(e.extensions);
  this.astNode = e.astNode;
  this.extensionASTNodes = null !== (i = e.extensionASTNodes) && void 0 !== i ? i : [];
  this._queryType = e.query;
  this._mutationType = e.mutation;
  this._subscriptionType = e.subscription;
  this._directives = null !== (a = e.directives) && void 0 !== a ? a : n;
  var c = new Set(e.types);
  if (null != e.types) {
    for (var l = 0, u = e.types; l < u.length; l += 1) {
      var y = u[l];
      c.delete(y);
      collectReferencedTypes(y, c);
    }
  }
  if (null != this._queryType) {
    collectReferencedTypes(this._queryType, c);
  }
  if (null != this._mutationType) {
    collectReferencedTypes(this._mutationType, c);
  }
  if (null != this._subscriptionType) {
    collectReferencedTypes(this._subscriptionType, c);
  }
  for (var f = 0, h = this._directives; f < h.length; f += 1) {
    var m = h[f];
    if (r(m)) {
      for (var v = 0, d = m.args; v < d.length; v += 1) {
        collectReferencedTypes(d[v].type, c);
      }
    }
  }
  collectReferencedTypes(s, c);
  this._typeMap = Object.create(null);
  this._subTypeMap = Object.create(null);
  this._implementationsMap = Object.create(null);
  for (var g = 0, T = c; g < T.length; g += 1) {
    var b = T[g];
    if (null == b) {
      continue;
    }
    var _ = b.name;
    if (void 0 !== this._typeMap[_]) {
      throw new Error('Schema must contain uniquely named types but contains multiple types named "' + _ + '".');
    }
    this._typeMap[_] = b;
    if (o(b)) {
      for (var j = 0, M = b.getInterfaces(); j < M.length; j += 1) {
        var S = M[j];
        if (o(S)) {
          var R = this._implementationsMap[S.name];
          if (void 0 === R) {
            R = this._implementationsMap[S.name] = {
              objects: [],
              interfaces: []
            };
          }
          R.interfaces.push(b);
        }
      }
    } else if (p(b)) {
      for (var x = 0, O = b.getInterfaces(); x < O.length; x += 1) {
        var N = O[x];
        if (o(N)) {
          var q = this._implementationsMap[N.name];
          if (void 0 === q) {
            q = this._implementationsMap[N.name] = {
              objects: [],
              interfaces: []
            };
          }
          q.objects.push(b);
        }
      }
    }
  }
};

u.prototype.getQueryType = function getQueryType() {
  return this._queryType;
};

u.prototype.getMutationType = function getMutationType() {
  return this._mutationType;
};

u.prototype.getSubscriptionType = function getSubscriptionType() {
  return this._subscriptionType;
};

u.prototype.getTypeMap = function getTypeMap() {
  return this._typeMap;
};

u.prototype.getType = function getType(e) {
  return this.getTypeMap()[e];
};

u.prototype.getPossibleTypes = function getPossibleTypes(e) {
  return a(e) ? e.getTypes() : this.getImplementations(e).objects;
};

u.prototype.getImplementations = function getImplementations(e) {
  var t = this._implementationsMap[e.name];
  return null != t ? t : {
    objects: [],
    interfaces: []
  };
};

u.prototype.isSubType = function isSubType(e, t) {
  var i = this._subTypeMap[e.name];
  if (void 0 === i) {
    i = Object.create(null);
    if (a(e)) {
      for (var s = 0, n = e.getTypes(); s < n.length; s += 1) {
        i[n[s].name] = !0;
      }
    } else {
      var r = this.getImplementations(e);
      for (var o = 0, p = r.objects; o < p.length; o += 1) {
        i[p[o].name] = !0;
      }
      for (var c = 0, l = r.interfaces; c < l.length; c += 1) {
        i[l[c].name] = !0;
      }
    }
    this._subTypeMap[e.name] = i;
  }
  return void 0 !== i[t.name];
};

u.prototype.getDirectives = function getDirectives() {
  return this._directives;
};

u.prototype.getDirective = function getDirective(e) {
  return this.getDirectives().find((function(t) {
    return t.name === e;
  }));
};

u.prototype.toConfig = function toConfig() {
  return {
    description: this.description,
    query: this.getQueryType(),
    mutation: this.getMutationType(),
    subscription: this.getSubscriptionType(),
    types: Object.values(this.getTypeMap()),
    directives: this.getDirectives(),
    extensions: this.extensions,
    astNode: this.astNode,
    extensionASTNodes: this.extensionASTNodes,
    assumeValid: void 0 !== this.__validationErrors
  };
};

function collectReferencedTypes(e, t) {
  var i = c(e);
  if (!t.has(i)) {
    t.add(i);
    if (a(i)) {
      for (var s = 0, n = i.getTypes(); s < n.length; s += 1) {
        collectReferencedTypes(n[s], t);
      }
    } else if (p(i) || o(i)) {
      for (var r = 0, u = i.getInterfaces(); r < u.length; r += 1) {
        collectReferencedTypes(u[r], t);
      }
      for (var y = 0, f = Object.values(i.getFields()); y < f.length; y += 1) {
        var h = f[y];
        collectReferencedTypes(h.type, t);
        for (var m = 0, v = h.args; m < v.length; m += 1) {
          collectReferencedTypes(v[m].type, t);
        }
      }
    } else if (l(i)) {
      for (var d = 0, g = Object.values(i.getFields()); d < g.length; d += 1) {
        collectReferencedTypes(g[d].type, t);
      }
    }
  }
  return t;
}

export { u as GraphQLSchema, assertSchema };
