"use strict";

Object.defineProperty(exports, "__esModule", {
  value: !0
});

var e = require("../jsutils/inspect.js");

var t = require("../error/GraphQLError.js");

var r = require("../error/locatedError.js");

var n = require("../utilities/assertValidName.js");

var a = require("../utilities/typeComparators.js");

var i = require("./schema.js");

var o = require("./introspection.js");

var s = require("./directives.js");

var p = require("./definition.js");

function validateSchema(t) {
  i.assertSchema(t);
  if (t.__validationErrors) {
    return t.__validationErrors;
  }
  var r = new l(t);
  !function validateRootTypes(t) {
    var r = t.schema;
    var n = r.getQueryType();
    if (!n) {
      t.reportError("Query root type must be provided.", r.astNode);
    } else if (!p.isObjectType(n)) {
      var a;
      t.reportError("Query root type must be Object type, it cannot be " + e.inspect(n) + ".", null !== (a = getOperationTypeNode(r, "query")) && void 0 !== a ? a : n.astNode);
    }
    var i = r.getMutationType();
    if (i && !p.isObjectType(i)) {
      var o;
      t.reportError("Mutation root type must be Object type if provided, it cannot be " + e.inspect(i) + ".", null !== (o = getOperationTypeNode(r, "mutation")) && void 0 !== o ? o : i.astNode);
    }
    var s = r.getSubscriptionType();
    if (s && !p.isObjectType(s)) {
      var l;
      t.reportError("Subscription root type must be Object type if provided, it cannot be " + e.inspect(s) + ".", null !== (l = getOperationTypeNode(r, "subscription")) && void 0 !== l ? l : s.astNode);
    }
  }(r);
  !function validateDirectives(t) {
    for (var r = 0, n = t.schema.getDirectives(); r < n.length; r += 1) {
      var a = n[r];
      if (!s.isDirective(a)) {
        t.reportError("Expected directive but got: " + e.inspect(a) + ".", null == a ? void 0 : a.astNode);
        continue;
      }
      validateName(t, a);
      for (var i = 0, o = a.args; i < o.length; i += 1) {
        var l = o[i];
        validateName(t, l);
        if (!p.isInputType(l.type)) {
          t.reportError("The type of @" + a.name + "(" + l.name + ":) must be Input Type but got: " + e.inspect(l.type) + ".", l.astNode);
        }
        if (p.isRequiredArgument(l) && null != l.deprecationReason) {
          var d;
          t.reportError("Required argument @" + a.name + "(" + l.name + ":) cannot be deprecated.", [ getDeprecatedDirectiveNode(l.astNode), null === (d = l.astNode) || void 0 === d ? void 0 : d.type ]);
        }
      }
    }
  }(r);
  !function validateTypes(t) {
    var r = function createInputObjectCircularRefsValidator(e) {
      var t = Object.create(null);
      var r = [];
      var n = Object.create(null);
      return detectCycleRecursive;
      function detectCycleRecursive(a) {
        if (t[a.name]) {
          return;
        }
        t[a.name] = !0;
        n[a.name] = r.length;
        for (var i = 0, o = Object.values(a.getFields()); i < o.length; i += 1) {
          var s = o[i];
          if (p.isNonNullType(s.type) && p.isInputObjectType(s.type.ofType)) {
            var l = s.type.ofType;
            var d = n[l.name];
            r.push(s);
            if (void 0 === d) {
              detectCycleRecursive(l);
            } else {
              var c = r.slice(d);
              var u = c.map((function(e) {
                return e.name;
              })).join(".");
              e.reportError('Cannot reference Input Object "' + l.name + '" within itself through a series of non-null fields: "' + u + '".', c.map((function(e) {
                return e.astNode;
              })));
            }
            r.pop();
          }
        }
        n[a.name] = void 0;
      }
    }(t);
    var n = t.schema.getTypeMap();
    for (var a = 0, i = Object.values(n); a < i.length; a += 1) {
      var s = i[a];
      if (!p.isNamedType(s)) {
        t.reportError("Expected GraphQL named type but got: " + e.inspect(s) + ".", s.astNode);
        continue;
      }
      if (!o.isIntrospectionType(s)) {
        validateName(t, s);
      }
      if (p.isObjectType(s)) {
        validateFields(t, s);
        validateInterfaces(t, s);
      } else if (p.isInterfaceType(s)) {
        validateFields(t, s);
        validateInterfaces(t, s);
      } else if (p.isUnionType(s)) {
        validateUnionMembers(t, s);
      } else if (p.isEnumType(s)) {
        validateEnumValues(t, s);
      } else if (p.isInputObjectType(s)) {
        validateInputFields(t, s);
        r(s);
      }
    }
  }(r);
  var n = r.getErrors();
  t.__validationErrors = n;
  return n;
}

var l = function SchemaValidationContext(e) {
  this._errors = [];
  this.schema = e;
};

l.prototype.reportError = function reportError(e, r) {
  var n = Array.isArray(r) ? r.filter(Boolean) : r;
  this.addError(new t.GraphQLError(e, n));
};

l.prototype.addError = function addError(e) {
  this._errors.push(e);
};

l.prototype.getErrors = function getErrors() {
  return this._errors;
};

function getOperationTypeNode(e, t) {
  var r;
  return null === (r = [ e.astNode ].concat(e.extensionASTNodes).flatMap((function(e) {
    var t;
    return null !== (t = null == e ? void 0 : e.operationTypes) && void 0 !== t ? t : [];
  })).find((function(e) {
    return e.operation === t;
  }))) || void 0 === r ? void 0 : r.type;
}

function validateName(e, t) {
  var a = n.isValidNameError(t.name);
  if (a) {
    e.addError(r.locatedError(a, t.astNode));
  }
}

function validateFields(t, r) {
  var n = Object.values(r.getFields());
  if (0 === n.length) {
    t.reportError("Type " + r.name + " must define one or more fields.", [ r.astNode ].concat(r.extensionASTNodes));
  }
  for (var a = 0, i = n; a < i.length; a += 1) {
    var o = i[a];
    validateName(t, o);
    if (!p.isOutputType(o.type)) {
      var s;
      t.reportError("The type of " + r.name + "." + o.name + " must be Output Type but got: " + e.inspect(o.type) + ".", null === (s = o.astNode) || void 0 === s ? void 0 : s.type);
    }
    for (var l = 0, d = o.args; l < d.length; l += 1) {
      var c = d[l];
      var u = c.name;
      validateName(t, c);
      if (!p.isInputType(c.type)) {
        var v;
        t.reportError("The type of " + r.name + "." + o.name + "(" + u + ":) must be Input Type but got: " + e.inspect(c.type) + ".", null === (v = c.astNode) || void 0 === v ? void 0 : v.type);
      }
      if (p.isRequiredArgument(c) && null != c.deprecationReason) {
        var m;
        t.reportError("Required argument " + r.name + "." + o.name + "(" + u + ":) cannot be deprecated.", [ getDeprecatedDirectiveNode(c.astNode), null === (m = c.astNode) || void 0 === m ? void 0 : m.type ]);
      }
    }
  }
}

function validateInterfaces(t, r) {
  var n = Object.create(null);
  for (var a = 0, i = r.getInterfaces(); a < i.length; a += 1) {
    var o = i[a];
    if (!p.isInterfaceType(o)) {
      t.reportError("Type " + e.inspect(r) + " must only implement Interface types, it cannot implement " + e.inspect(o) + ".", getAllImplementsInterfaceNodes(r, o));
      continue;
    }
    if (r === o) {
      t.reportError("Type " + r.name + " cannot implement itself because it would create a circular reference.", getAllImplementsInterfaceNodes(r, o));
      continue;
    }
    if (n[o.name]) {
      t.reportError("Type " + r.name + " can only implement " + o.name + " once.", getAllImplementsInterfaceNodes(r, o));
      continue;
    }
    n[o.name] = !0;
    validateTypeImplementsAncestors(t, r, o);
    validateTypeImplementsInterface(t, r, o);
  }
}

function validateTypeImplementsInterface(t, r, n) {
  var i = r.getFields();
  for (var o = 0, s = Object.values(n.getFields()); o < s.length; o += 1) {
    var l = s[o];
    var d = l.name;
    var c = i[d];
    if (!c) {
      t.reportError("Interface field " + n.name + "." + d + " expected but " + r.name + " does not provide it.", [ l.astNode, r.astNode ].concat(r.extensionASTNodes));
      continue;
    }
    if (!a.isTypeSubTypeOf(t.schema, c.type, l.type)) {
      var u, v;
      t.reportError("Interface field " + n.name + "." + d + " expects type " + e.inspect(l.type) + " but " + r.name + "." + d + " is type " + e.inspect(c.type) + ".", [ null === (u = l.astNode) || void 0 === u ? void 0 : u.type, null === (v = c.astNode) || void 0 === v ? void 0 : v.type ]);
    }
    var loop = function() {
      var i = f[m];
      var o = i.name;
      var s = c.args.find((function(e) {
        return e.name === o;
      }));
      if (!s) {
        t.reportError("Interface field argument " + n.name + "." + d + "(" + o + ":) expected but " + r.name + "." + d + " does not provide it.", [ i.astNode, c.astNode ]);
        return;
      }
      if (!a.isEqualType(i.type, s.type)) {
        var p, l;
        t.reportError("Interface field argument " + n.name + "." + d + "(" + o + ":) expects type " + e.inspect(i.type) + " but " + r.name + "." + d + "(" + o + ":) is type " + e.inspect(s.type) + ".", [ null === (p = i.astNode) || void 0 === p ? void 0 : p.type, null === (l = s.astNode) || void 0 === l ? void 0 : l.type ]);
      }
    };
    for (var m = 0, f = l.args; m < f.length; m += 1) {
      loop();
    }
    var loop$1 = function() {
      var e = g[y];
      var a = e.name;
      if (!l.args.find((function(e) {
        return e.name === a;
      })) && p.isRequiredArgument(e)) {
        t.reportError("Object field " + r.name + "." + d + " includes required argument " + a + " that is missing from the Interface field " + n.name + "." + d + ".", [ e.astNode, l.astNode ]);
      }
    };
    for (var y = 0, g = c.args; y < g.length; y += 1) {
      loop$1();
    }
  }
}

function validateTypeImplementsAncestors(e, t, r) {
  var n = t.getInterfaces();
  for (var a = 0, i = r.getInterfaces(); a < i.length; a += 1) {
    var o = i[a];
    if (!n.includes(o)) {
      e.reportError(o === t ? "Type " + t.name + " cannot implement " + r.name + " because it would create a circular reference." : "Type " + t.name + " must implement " + o.name + " because it is implemented by " + r.name + ".", getAllImplementsInterfaceNodes(r, o).concat(getAllImplementsInterfaceNodes(t, r)));
    }
  }
}

function validateUnionMembers(t, r) {
  var n = r.getTypes();
  if (0 === n.length) {
    t.reportError("Union type " + r.name + " must define one or more member types.", [ r.astNode ].concat(r.extensionASTNodes));
  }
  var a = Object.create(null);
  for (var i = 0, o = n; i < o.length; i += 1) {
    var s = o[i];
    if (a[s.name]) {
      t.reportError("Union type " + r.name + " can only include type " + s.name + " once.", getUnionMemberTypeNodes(r, s.name));
      continue;
    }
    a[s.name] = !0;
    if (!p.isObjectType(s)) {
      t.reportError("Union type " + r.name + " can only include Object types, it cannot include " + e.inspect(s) + ".", getUnionMemberTypeNodes(r, String(s)));
    }
  }
}

function validateEnumValues(e, t) {
  var r = t.getValues();
  if (0 === r.length) {
    e.reportError("Enum type " + t.name + " must define one or more values.", [ t.astNode ].concat(t.extensionASTNodes));
  }
  for (var n = 0, a = r; n < a.length; n += 1) {
    var i = a[n];
    var o = i.name;
    validateName(e, i);
    if ("true" === o || "false" === o || "null" === o) {
      e.reportError("Enum type " + t.name + " cannot include value: " + o + ".", i.astNode);
    }
  }
}

function validateInputFields(t, r) {
  var n = Object.values(r.getFields());
  if (0 === n.length) {
    t.reportError("Input Object type " + r.name + " must define one or more fields.", [ r.astNode ].concat(r.extensionASTNodes));
  }
  for (var a = 0, i = n; a < i.length; a += 1) {
    var o = i[a];
    validateName(t, o);
    if (!p.isInputType(o.type)) {
      var s;
      t.reportError("The type of " + r.name + "." + o.name + " must be Input Type but got: " + e.inspect(o.type) + ".", null === (s = o.astNode) || void 0 === s ? void 0 : s.type);
    }
    if (p.isRequiredInputField(o) && null != o.deprecationReason) {
      var l;
      t.reportError("Required input field " + r.name + "." + o.name + " cannot be deprecated.", [ getDeprecatedDirectiveNode(o.astNode), null === (l = o.astNode) || void 0 === l ? void 0 : l.type ]);
    }
  }
}

function getAllImplementsInterfaceNodes(e, t) {
  var r = e.astNode;
  var n = e.extensionASTNodes;
  return (null != r ? [ r ].concat(n) : n).flatMap((function(e) {
    var t;
    return null !== (t = e.interfaces) && void 0 !== t ? t : [];
  })).filter((function(e) {
    return e.name.value === t.name;
  }));
}

function getUnionMemberTypeNodes(e, t) {
  var r = e.astNode;
  var n = e.extensionASTNodes;
  return (null != r ? [ r ].concat(n) : n).flatMap((function(e) {
    var t;
    return null !== (t = e.types) && void 0 !== t ? t : [];
  })).filter((function(e) {
    return e.name.value === t;
  }));
}

function getDeprecatedDirectiveNode(e) {
  var t;
  return null == e ? void 0 : null === (t = e.directives) || void 0 === t ? void 0 : t.find((function(e) {
    return e.name.value === s.GraphQLDeprecatedDirective.name;
  }));
}

exports.assertValidSchema = function assertValidSchema(e) {
  var t = validateSchema(e);
  if (0 !== t.length) {
    throw new Error(t.map((function(e) {
      return e.message;
    })).join("\n\n"));
  }
};

exports.validateSchema = validateSchema;
