import { inspect as e } from "../jsutils/inspect.mjs";

import { GraphQLError as t } from "../error/GraphQLError.mjs";

import { locatedError as r } from "../error/locatedError.mjs";

import { isValidNameError as n } from "../utilities/assertValidName.mjs";

import { isTypeSubTypeOf as a, isEqualType as o } from "../utilities/typeComparators.mjs";

import { assertSchema as i } from "./schema.mjs";

import { isIntrospectionType as l } from "./introspection.mjs";

import { isDirective as s, GraphQLDeprecatedDirective as d } from "./directives.mjs";

import { isObjectType as p, isInputType as c, isRequiredArgument as u, isNamedType as m, isInterfaceType as v, isUnionType as f, isEnumType as y, isInputObjectType as g, isOutputType as N, isRequiredInputField as b, isNonNullType as h } from "./definition.mjs";

function validateSchema(t) {
  i(t);
  if (t.__validationErrors) {
    return t.__validationErrors;
  }
  var r = new E(t);
  !function validateRootTypes(t) {
    var r = t.schema;
    var n = r.getQueryType();
    if (!n) {
      t.reportError("Query root type must be provided.", r.astNode);
    } else if (!p(n)) {
      var a;
      t.reportError("Query root type must be Object type, it cannot be " + e(n) + ".", null !== (a = getOperationTypeNode(r, "query")) && void 0 !== a ? a : n.astNode);
    }
    var o = r.getMutationType();
    if (o && !p(o)) {
      var i;
      t.reportError("Mutation root type must be Object type if provided, it cannot be " + e(o) + ".", null !== (i = getOperationTypeNode(r, "mutation")) && void 0 !== i ? i : o.astNode);
    }
    var l = r.getSubscriptionType();
    if (l && !p(l)) {
      var s;
      t.reportError("Subscription root type must be Object type if provided, it cannot be " + e(l) + ".", null !== (s = getOperationTypeNode(r, "subscription")) && void 0 !== s ? s : l.astNode);
    }
  }(r);
  !function validateDirectives(t) {
    for (var r = 0, n = t.schema.getDirectives(); r < n.length; r += 1) {
      var a = n[r];
      if (!s(a)) {
        t.reportError("Expected directive but got: " + e(a) + ".", null == a ? void 0 : a.astNode);
        continue;
      }
      validateName(t, a);
      for (var o = 0, i = a.args; o < i.length; o += 1) {
        var l = i[o];
        validateName(t, l);
        if (!c(l.type)) {
          t.reportError("The type of @" + a.name + "(" + l.name + ":) must be Input Type but got: " + e(l.type) + ".", l.astNode);
        }
        if (u(l) && null != l.deprecationReason) {
          var d;
          t.reportError("Required argument @" + a.name + "(" + l.name + ":) cannot be deprecated.", [ getDeprecatedDirectiveNode(l.astNode), null === (d = l.astNode) || void 0 === d ? void 0 : d.type ]);
        }
      }
    }
  }(r);
  !function validateTypes(t) {
    var r = function createInputObjectCircularRefsValidator(e) {
      var t = Object.create(null);
      var r = [];
      var n = Object.create(null);
      return detectCycleRecursive;
      function detectCycleRecursive(a) {
        if (t[a.name]) {
          return;
        }
        t[a.name] = !0;
        n[a.name] = r.length;
        for (var o = 0, i = Object.values(a.getFields()); o < i.length; o += 1) {
          var l = i[o];
          if (h(l.type) && g(l.type.ofType)) {
            var s = l.type.ofType;
            var d = n[s.name];
            r.push(l);
            if (void 0 === d) {
              detectCycleRecursive(s);
            } else {
              var p = r.slice(d);
              var c = p.map((function(e) {
                return e.name;
              })).join(".");
              e.reportError('Cannot reference Input Object "' + s.name + '" within itself through a series of non-null fields: "' + c + '".', p.map((function(e) {
                return e.astNode;
              })));
            }
            r.pop();
          }
        }
        n[a.name] = void 0;
      }
    }(t);
    var n = t.schema.getTypeMap();
    for (var a = 0, o = Object.values(n); a < o.length; a += 1) {
      var i = o[a];
      if (!m(i)) {
        t.reportError("Expected GraphQL named type but got: " + e(i) + ".", i.astNode);
        continue;
      }
      if (!l(i)) {
        validateName(t, i);
      }
      if (p(i)) {
        validateFields(t, i);
        validateInterfaces(t, i);
      } else if (v(i)) {
        validateFields(t, i);
        validateInterfaces(t, i);
      } else if (f(i)) {
        validateUnionMembers(t, i);
      } else if (y(i)) {
        validateEnumValues(t, i);
      } else if (g(i)) {
        validateInputFields(t, i);
        r(i);
      }
    }
  }(r);
  var n = r.getErrors();
  t.__validationErrors = n;
  return n;
}

function assertValidSchema(e) {
  var t = validateSchema(e);
  if (0 !== t.length) {
    throw new Error(t.map((function(e) {
      return e.message;
    })).join("\n\n"));
  }
}

var E = function SchemaValidationContext(e) {
  this._errors = [];
  this.schema = e;
};

E.prototype.reportError = function reportError(e, r) {
  var n = Array.isArray(r) ? r.filter(Boolean) : r;
  this.addError(new t(e, n));
};

E.prototype.addError = function addError(e) {
  this._errors.push(e);
};

E.prototype.getErrors = function getErrors() {
  return this._errors;
};

function getOperationTypeNode(e, t) {
  var r;
  return null === (r = [ e.astNode ].concat(e.extensionASTNodes).flatMap((function(e) {
    var t;
    return null !== (t = null == e ? void 0 : e.operationTypes) && void 0 !== t ? t : [];
  })).find((function(e) {
    return e.operation === t;
  }))) || void 0 === r ? void 0 : r.type;
}

function validateName(e, t) {
  var a = n(t.name);
  if (a) {
    e.addError(r(a, t.astNode));
  }
}

function validateFields(t, r) {
  var n = Object.values(r.getFields());
  if (0 === n.length) {
    t.reportError("Type " + r.name + " must define one or more fields.", [ r.astNode ].concat(r.extensionASTNodes));
  }
  for (var a = 0, o = n; a < o.length; a += 1) {
    var i = o[a];
    validateName(t, i);
    if (!N(i.type)) {
      var l;
      t.reportError("The type of " + r.name + "." + i.name + " must be Output Type but got: " + e(i.type) + ".", null === (l = i.astNode) || void 0 === l ? void 0 : l.type);
    }
    for (var s = 0, d = i.args; s < d.length; s += 1) {
      var p = d[s];
      var m = p.name;
      validateName(t, p);
      if (!c(p.type)) {
        var v;
        t.reportError("The type of " + r.name + "." + i.name + "(" + m + ":) must be Input Type but got: " + e(p.type) + ".", null === (v = p.astNode) || void 0 === v ? void 0 : v.type);
      }
      if (u(p) && null != p.deprecationReason) {
        var f;
        t.reportError("Required argument " + r.name + "." + i.name + "(" + m + ":) cannot be deprecated.", [ getDeprecatedDirectiveNode(p.astNode), null === (f = p.astNode) || void 0 === f ? void 0 : f.type ]);
      }
    }
  }
}

function validateInterfaces(t, r) {
  var n = Object.create(null);
  for (var a = 0, o = r.getInterfaces(); a < o.length; a += 1) {
    var i = o[a];
    if (!v(i)) {
      t.reportError("Type " + e(r) + " must only implement Interface types, it cannot implement " + e(i) + ".", getAllImplementsInterfaceNodes(r, i));
      continue;
    }
    if (r === i) {
      t.reportError("Type " + r.name + " cannot implement itself because it would create a circular reference.", getAllImplementsInterfaceNodes(r, i));
      continue;
    }
    if (n[i.name]) {
      t.reportError("Type " + r.name + " can only implement " + i.name + " once.", getAllImplementsInterfaceNodes(r, i));
      continue;
    }
    n[i.name] = !0;
    validateTypeImplementsAncestors(t, r, i);
    validateTypeImplementsInterface(t, r, i);
  }
}

function validateTypeImplementsInterface(t, r, n) {
  var i = r.getFields();
  for (var l = 0, s = Object.values(n.getFields()); l < s.length; l += 1) {
    var d = s[l];
    var p = d.name;
    var c = i[p];
    if (!c) {
      t.reportError("Interface field " + n.name + "." + p + " expected but " + r.name + " does not provide it.", [ d.astNode, r.astNode ].concat(r.extensionASTNodes));
      continue;
    }
    if (!a(t.schema, c.type, d.type)) {
      var m, v;
      t.reportError("Interface field " + n.name + "." + p + " expects type " + e(d.type) + " but " + r.name + "." + p + " is type " + e(c.type) + ".", [ null === (m = d.astNode) || void 0 === m ? void 0 : m.type, null === (v = c.astNode) || void 0 === v ? void 0 : v.type ]);
    }
    var loop = function() {
      var a = y[f];
      var i = a.name;
      var l = c.args.find((function(e) {
        return e.name === i;
      }));
      if (!l) {
        t.reportError("Interface field argument " + n.name + "." + p + "(" + i + ":) expected but " + r.name + "." + p + " does not provide it.", [ a.astNode, c.astNode ]);
        return;
      }
      if (!o(a.type, l.type)) {
        var s, d;
        t.reportError("Interface field argument " + n.name + "." + p + "(" + i + ":) expects type " + e(a.type) + " but " + r.name + "." + p + "(" + i + ":) is type " + e(l.type) + ".", [ null === (s = a.astNode) || void 0 === s ? void 0 : s.type, null === (d = l.astNode) || void 0 === d ? void 0 : d.type ]);
      }
    };
    for (var f = 0, y = d.args; f < y.length; f += 1) {
      loop();
    }
    var loop$1 = function() {
      var e = N[g];
      var a = e.name;
      if (!d.args.find((function(e) {
        return e.name === a;
      })) && u(e)) {
        t.reportError("Object field " + r.name + "." + p + " includes required argument " + a + " that is missing from the Interface field " + n.name + "." + p + ".", [ e.astNode, d.astNode ]);
      }
    };
    for (var g = 0, N = c.args; g < N.length; g += 1) {
      loop$1();
    }
  }
}

function validateTypeImplementsAncestors(e, t, r) {
  var n = t.getInterfaces();
  for (var a = 0, o = r.getInterfaces(); a < o.length; a += 1) {
    var i = o[a];
    if (!n.includes(i)) {
      e.reportError(i === t ? "Type " + t.name + " cannot implement " + r.name + " because it would create a circular reference." : "Type " + t.name + " must implement " + i.name + " because it is implemented by " + r.name + ".", getAllImplementsInterfaceNodes(r, i).concat(getAllImplementsInterfaceNodes(t, r)));
    }
  }
}

function validateUnionMembers(t, r) {
  var n = r.getTypes();
  if (0 === n.length) {
    t.reportError("Union type " + r.name + " must define one or more member types.", [ r.astNode ].concat(r.extensionASTNodes));
  }
  var a = Object.create(null);
  for (var o = 0, i = n; o < i.length; o += 1) {
    var l = i[o];
    if (a[l.name]) {
      t.reportError("Union type " + r.name + " can only include type " + l.name + " once.", getUnionMemberTypeNodes(r, l.name));
      continue;
    }
    a[l.name] = !0;
    if (!p(l)) {
      t.reportError("Union type " + r.name + " can only include Object types, it cannot include " + e(l) + ".", getUnionMemberTypeNodes(r, String(l)));
    }
  }
}

function validateEnumValues(e, t) {
  var r = t.getValues();
  if (0 === r.length) {
    e.reportError("Enum type " + t.name + " must define one or more values.", [ t.astNode ].concat(t.extensionASTNodes));
  }
  for (var n = 0, a = r; n < a.length; n += 1) {
    var o = a[n];
    var i = o.name;
    validateName(e, o);
    if ("true" === i || "false" === i || "null" === i) {
      e.reportError("Enum type " + t.name + " cannot include value: " + i + ".", o.astNode);
    }
  }
}

function validateInputFields(t, r) {
  var n = Object.values(r.getFields());
  if (0 === n.length) {
    t.reportError("Input Object type " + r.name + " must define one or more fields.", [ r.astNode ].concat(r.extensionASTNodes));
  }
  for (var a = 0, o = n; a < o.length; a += 1) {
    var i = o[a];
    validateName(t, i);
    if (!c(i.type)) {
      var l;
      t.reportError("The type of " + r.name + "." + i.name + " must be Input Type but got: " + e(i.type) + ".", null === (l = i.astNode) || void 0 === l ? void 0 : l.type);
    }
    if (b(i) && null != i.deprecationReason) {
      var s;
      t.reportError("Required input field " + r.name + "." + i.name + " cannot be deprecated.", [ getDeprecatedDirectiveNode(i.astNode), null === (s = i.astNode) || void 0 === s ? void 0 : s.type ]);
    }
  }
}

function getAllImplementsInterfaceNodes(e, t) {
  var r = e.astNode;
  var n = e.extensionASTNodes;
  return (null != r ? [ r ].concat(n) : n).flatMap((function(e) {
    var t;
    return null !== (t = e.interfaces) && void 0 !== t ? t : [];
  })).filter((function(e) {
    return e.name.value === t.name;
  }));
}

function getUnionMemberTypeNodes(e, t) {
  var r = e.astNode;
  var n = e.extensionASTNodes;
  return (null != r ? [ r ].concat(n) : n).flatMap((function(e) {
    var t;
    return null !== (t = e.types) && void 0 !== t ? t : [];
  })).filter((function(e) {
    return e.name.value === t;
  }));
}

function getDeprecatedDirectiveNode(e) {
  var t;
  return null == e ? void 0 : null === (t = e.directives) || void 0 === t ? void 0 : t.find((function(e) {
    return e.name.value === d.name;
  }));
}

export { assertValidSchema, validateSchema };
