import { Kind as t } from "../language/kinds.mjs";

import { isNode as e } from "../language/ast.mjs";

import { getVisitFn as i } from "../language/visitor.mjs";

import { isInputType as a, isCompositeType as p, isOutputType as n, getNamedType as r, isEnumType as u, isInputObjectType as s, getNullableType as h, isListType as c, isObjectType as l, isInterfaceType as o } from "../type/definition.mjs";

import { SchemaMetaFieldDef as f, TypeMetaFieldDef as y, TypeNameMetaFieldDef as _ } from "../type/introspection.mjs";

import { typeFromAST as k } from "./typeFromAST.mjs";

var T = function TypeInfo(t, e, i) {
  this._schema = t;
  this._typeStack = [];
  this._parentTypeStack = [];
  this._inputTypeStack = [];
  this._fieldDefStack = [];
  this._defaultValueStack = [];
  this._directive = null;
  this._argument = null;
  this._enumValue = null;
  this._getFieldDef = null != i ? i : getFieldDef;
  if (e) {
    if (a(e)) {
      this._inputTypeStack.push(e);
    }
    if (p(e)) {
      this._parentTypeStack.push(e);
    }
    if (n(e)) {
      this._typeStack.push(e);
    }
  }
};

T.prototype.getType = function getType() {
  if (this._typeStack.length > 0) {
    return this._typeStack[this._typeStack.length - 1];
  }
};

T.prototype.getParentType = function getParentType() {
  if (this._parentTypeStack.length > 0) {
    return this._parentTypeStack[this._parentTypeStack.length - 1];
  }
};

T.prototype.getInputType = function getInputType() {
  if (this._inputTypeStack.length > 0) {
    return this._inputTypeStack[this._inputTypeStack.length - 1];
  }
};

T.prototype.getParentInputType = function getParentInputType() {
  if (this._inputTypeStack.length > 1) {
    return this._inputTypeStack[this._inputTypeStack.length - 2];
  }
};

T.prototype.getFieldDef = function getFieldDef() {
  if (this._fieldDefStack.length > 0) {
    return this._fieldDefStack[this._fieldDefStack.length - 1];
  }
};

T.prototype.getDefaultValue = function getDefaultValue() {
  if (this._defaultValueStack.length > 0) {
    return this._defaultValueStack[this._defaultValueStack.length - 1];
  }
};

T.prototype.getDirective = function getDirective() {
  return this._directive;
};

T.prototype.getArgument = function getArgument() {
  return this._argument;
};

T.prototype.getEnumValue = function getEnumValue() {
  return this._enumValue;
};

T.prototype.enter = function enter(e) {
  var i = this._schema;
  switch (e.kind) {
   case t.SELECTION_SET:
    var o = r(this.getType());
    this._parentTypeStack.push(p(o) ? o : void 0);
    break;

   case t.FIELD:
    var f = this.getParentType();
    var y;
    var _;
    if (f) {
      if (y = this._getFieldDef(i, f, e)) {
        _ = y.type;
      }
    }
    this._fieldDefStack.push(y);
    this._typeStack.push(n(_) ? _ : void 0);
    break;

   case t.DIRECTIVE:
    this._directive = i.getDirective(e.name.value);
    break;

   case t.OPERATION_DEFINITION:
    var T;
    switch (e.operation) {
     case "query":
      T = i.getQueryType();
      break;

     case "mutation":
      T = i.getMutationType();
      break;

     case "subscription":
      T = i.getSubscriptionType();
    }
    this._typeStack.push(l(T) ? T : void 0);
    break;

   case t.INLINE_FRAGMENT:
   case t.FRAGMENT_DEFINITION:
    var g = e.typeCondition;
    var v = g ? k(i, g) : r(this.getType());
    this._typeStack.push(n(v) ? v : void 0);
    break;

   case t.VARIABLE_DEFINITION:
    var S = k(i, e.type);
    this._inputTypeStack.push(a(S) ? S : void 0);
    break;

   case t.ARGUMENT:
    var d;
    var I;
    var m;
    var E = null !== (d = this.getDirective()) && void 0 !== d ? d : this.getFieldDef();
    if (E) {
      if (I = E.args.find((function(t) {
        return t.name === e.name.value;
      }))) {
        m = I.type;
      }
    }
    this._argument = I;
    this._defaultValueStack.push(I ? I.defaultValue : void 0);
    this._inputTypeStack.push(a(m) ? m : void 0);
    break;

   case t.LIST:
    var D = h(this.getInputType());
    var N = c(D) ? D.ofType : D;
    this._defaultValueStack.push(void 0);
    this._inputTypeStack.push(a(N) ? N : void 0);
    break;

   case t.OBJECT_FIELD:
    var F = r(this.getInputType());
    var V;
    var b;
    if (s(F)) {
      if (b = F.getFields()[e.name.value]) {
        V = b.type;
      }
    }
    this._defaultValueStack.push(b ? b.defaultValue : void 0);
    this._inputTypeStack.push(a(V) ? V : void 0);
    break;

   case t.ENUM:
    var A = r(this.getInputType());
    var O;
    if (u(A)) {
      O = A.getValue(e.value);
    }
    this._enumValue = O;
  }
};

T.prototype.leave = function leave(e) {
  switch (e.kind) {
   case t.SELECTION_SET:
    this._parentTypeStack.pop();
    break;

   case t.FIELD:
    this._fieldDefStack.pop();
    this._typeStack.pop();
    break;

   case t.DIRECTIVE:
    this._directive = null;
    break;

   case t.OPERATION_DEFINITION:
   case t.INLINE_FRAGMENT:
   case t.FRAGMENT_DEFINITION:
    this._typeStack.pop();
    break;

   case t.VARIABLE_DEFINITION:
    this._inputTypeStack.pop();
    break;

   case t.ARGUMENT:
    this._argument = null;
    this._defaultValueStack.pop();
    this._inputTypeStack.pop();
    break;

   case t.LIST:
   case t.OBJECT_FIELD:
    this._defaultValueStack.pop();
    this._inputTypeStack.pop();
    break;

   case t.ENUM:
    this._enumValue = null;
  }
};

function getFieldDef(t, e, i) {
  var a = i.name.value;
  if (a === f.name && t.getQueryType() === e) {
    return f;
  }
  if (a === y.name && t.getQueryType() === e) {
    return y;
  }
  if (a === _.name && p(e)) {
    return _;
  }
  if (l(e) || o(e)) {
    return e.getFields()[a];
  }
}

function visitWithTypeInfo(t, a) {
  return {
    enter: function enter() {
      var p = [], n = arguments.length;
      while (n--) {
        p[n] = arguments[n];
      }
      var r = p[0];
      t.enter(r);
      var u = i(a, r.kind, !1);
      if (u) {
        var s = u.apply(a, p);
        if (void 0 !== s) {
          t.leave(r);
          if (e(s)) {
            t.enter(s);
          }
        }
        return s;
      }
    },
    leave: function leave() {
      var e = [], p = arguments.length;
      while (p--) {
        e[p] = arguments[p];
      }
      var n = e[0];
      var r = i(a, n.kind, !0);
      var u;
      if (r) {
        u = r.apply(a, e);
      }
      t.leave(n);
      return u;
    }
  };
}

export { T as TypeInfo, visitWithTypeInfo };
