"use strict";

Object.defineProperty(exports, "__esModule", {
  value: !0
});

var e = require("../jsutils/inspect.js");

var n = require("../jsutils/keyValMap.js");

var r = require("../language/parser.js");

var t = require("../type/schema.js");

var i = require("../type/directives.js");

var s = require("../type/scalars.js");

var a = require("../type/introspection.js");

var u = require("../type/definition.js");

var p = require("./valueFromAST.js");

exports.buildClientSchema = function buildClientSchema(o, c) {
  var l = o.__schema;
  var d = n.keyValMap(l.types, (function(e) {
    return e.name;
  }), (function(r) {
    return function buildType(r) {
      if (null != r && null != r.name && null != r.kind) {
        switch (r.kind) {
         case a.TypeKind.SCALAR:
          return function buildScalarDef(e) {
            return new u.GraphQLScalarType({
              name: e.name,
              description: e.description,
              specifiedByURL: e.specifiedByURL
            });
          }(r);

         case a.TypeKind.OBJECT:
          return function buildObjectDef(e) {
            return new u.GraphQLObjectType({
              name: e.name,
              description: e.description,
              interfaces: function() {
                return buildImplementationsList(e);
              },
              fields: function() {
                return buildFieldDefMap(e);
              }
            });
          }(r);

         case a.TypeKind.INTERFACE:
          return function buildInterfaceDef(e) {
            return new u.GraphQLInterfaceType({
              name: e.name,
              description: e.description,
              interfaces: function() {
                return buildImplementationsList(e);
              },
              fields: function() {
                return buildFieldDefMap(e);
              }
            });
          }(r);

         case a.TypeKind.UNION:
          return function buildUnionDef(n) {
            if (!n.possibleTypes) {
              var r = e.inspect(n);
              throw new Error("Introspection result missing possibleTypes: " + r + ".");
            }
            return new u.GraphQLUnionType({
              name: n.name,
              description: n.description,
              types: function() {
                return n.possibleTypes.map(getObjectType);
              }
            });
          }(r);

         case a.TypeKind.ENUM:
          return function buildEnumDef(r) {
            if (!r.enumValues) {
              var t = e.inspect(r);
              throw new Error("Introspection result missing enumValues: " + t + ".");
            }
            return new u.GraphQLEnumType({
              name: r.name,
              description: r.description,
              values: n.keyValMap(r.enumValues, (function(e) {
                return e.name;
              }), (function(e) {
                return {
                  description: e.description,
                  deprecationReason: e.deprecationReason
                };
              }))
            });
          }(r);

         case a.TypeKind.INPUT_OBJECT:
          return function buildInputObjectDef(n) {
            if (!n.inputFields) {
              var r = e.inspect(n);
              throw new Error("Introspection result missing inputFields: " + r + ".");
            }
            return new u.GraphQLInputObjectType({
              name: n.name,
              description: n.description,
              fields: function() {
                return buildInputValueDefMap(n.inputFields);
              }
            });
          }(r);
        }
      }
      var t = e.inspect(r);
      throw new Error("Invalid or incomplete introspection result. Ensure that a full introspection query is used in order to build a client schema: " + t + ".");
    }(r);
  }));
  for (var f = 0, y = s.specifiedScalarTypes.concat(a.introspectionTypes); f < y.length; f += 1) {
    var m = y[f];
    if (d[m.name]) {
      d[m.name] = m;
    }
  }
  var T = l.queryType ? getObjectType(l.queryType) : null;
  var v = l.mutationType ? getObjectType(l.mutationType) : null;
  var b = l.subscriptionType ? getObjectType(l.subscriptionType) : null;
  var w = l.directives ? l.directives.map((function buildDirective(n) {
    if (!n.args) {
      var r = e.inspect(n);
      throw new Error("Introspection result missing directive args: " + r + ".");
    }
    if (!n.locations) {
      var t = e.inspect(n);
      throw new Error("Introspection result missing directive locations: " + t + ".");
    }
    return new i.GraphQLDirective({
      name: n.name,
      description: n.description,
      isRepeatable: n.isRepeatable,
      locations: n.locations.slice(),
      args: buildInputValueDefMap(n.args)
    });
  })) : [];
  return new t.GraphQLSchema({
    description: l.description,
    query: T,
    mutation: v,
    subscription: b,
    types: Object.values(d),
    directives: w,
    assumeValid: null == c ? void 0 : c.assumeValid
  });
  function getType(e) {
    if (e.kind === a.TypeKind.LIST) {
      var n = e.ofType;
      if (!n) {
        throw new Error("Decorated type deeper than introspection query.");
      }
      return new u.GraphQLList(getType(n));
    }
    if (e.kind === a.TypeKind.NON_NULL) {
      var r = e.ofType;
      if (!r) {
        throw new Error("Decorated type deeper than introspection query.");
      }
      var t = getType(r);
      return new u.GraphQLNonNull(u.assertNullableType(t));
    }
    return getNamedType(e);
  }
  function getNamedType(n) {
    var r = n.name;
    if (!r) {
      throw new Error("Unknown type reference: " + e.inspect(n) + ".");
    }
    var t = d[r];
    if (!t) {
      throw new Error("Invalid or incomplete schema, unknown type: " + r + ". Ensure that a full introspection query is used in order to build a client schema.");
    }
    return t;
  }
  function getObjectType(e) {
    return u.assertObjectType(getNamedType(e));
  }
  function getInterfaceType(e) {
    return u.assertInterfaceType(getNamedType(e));
  }
  function buildImplementationsList(n) {
    if (null === n.interfaces && n.kind === a.TypeKind.INTERFACE) {
      return [];
    }
    if (!n.interfaces) {
      var r = e.inspect(n);
      throw new Error("Introspection result missing interfaces: " + r + ".");
    }
    return n.interfaces.map(getInterfaceType);
  }
  function buildFieldDefMap(r) {
    if (!r.fields) {
      throw new Error("Introspection result missing fields: " + e.inspect(r) + ".");
    }
    return n.keyValMap(r.fields, (function(e) {
      return e.name;
    }), buildField);
  }
  function buildField(n) {
    var r = getType(n.type);
    if (!u.isOutputType(r)) {
      var t = e.inspect(r);
      throw new Error("Introspection must provide output type for fields, but received: " + t + ".");
    }
    if (!n.args) {
      var i = e.inspect(n);
      throw new Error("Introspection result missing field args: " + i + ".");
    }
    return {
      description: n.description,
      deprecationReason: n.deprecationReason,
      type: r,
      args: buildInputValueDefMap(n.args)
    };
  }
  function buildInputValueDefMap(e) {
    return n.keyValMap(e, (function(e) {
      return e.name;
    }), buildInputValue);
  }
  function buildInputValue(n) {
    var t = getType(n.type);
    if (!u.isInputType(t)) {
      var i = e.inspect(t);
      throw new Error("Introspection must provide input type for arguments, but received: " + i + ".");
    }
    var s = null != n.defaultValue ? p.valueFromAST(r.parseValue(n.defaultValue), t) : void 0;
    return {
      description: n.description,
      type: t,
      defaultValue: s,
      deprecationReason: n.deprecationReason
    };
  }
};
