import { inspect as e } from "../jsutils/inspect.mjs";

import { keyValMap as n } from "../jsutils/keyValMap.mjs";

import { parseValue as r } from "../language/parser.mjs";

import { GraphQLSchema as t } from "../type/schema.mjs";

import { GraphQLDirective as i } from "../type/directives.mjs";

import { specifiedScalarTypes as o } from "../type/scalars.mjs";

import { introspectionTypes as s, TypeKind as u } from "../type/introspection.mjs";

import { assertObjectType as a, GraphQLInputObjectType as p, GraphQLEnumType as c, GraphQLUnionType as l, GraphQLInterfaceType as d, GraphQLObjectType as f, GraphQLScalarType as m, isOutputType as y, isInputType as b, GraphQLList as v, GraphQLNonNull as w, assertNullableType as g, assertInterfaceType as T } from "../type/definition.mjs";

import { valueFromAST as I } from "./valueFromAST.mjs";

function buildClientSchema(h, E) {
  var j = h.__schema;
  var D = n(j.types, (function(e) {
    return e.name;
  }), (function(r) {
    return function buildType(r) {
      if (null != r && null != r.name && null != r.kind) {
        switch (r.kind) {
         case u.SCALAR:
          return function buildScalarDef(e) {
            return new m({
              name: e.name,
              description: e.description,
              specifiedByURL: e.specifiedByURL
            });
          }(r);

         case u.OBJECT:
          return function buildObjectDef(e) {
            return new f({
              name: e.name,
              description: e.description,
              interfaces: function() {
                return buildImplementationsList(e);
              },
              fields: function() {
                return buildFieldDefMap(e);
              }
            });
          }(r);

         case u.INTERFACE:
          return function buildInterfaceDef(e) {
            return new d({
              name: e.name,
              description: e.description,
              interfaces: function() {
                return buildImplementationsList(e);
              },
              fields: function() {
                return buildFieldDefMap(e);
              }
            });
          }(r);

         case u.UNION:
          return function buildUnionDef(n) {
            if (!n.possibleTypes) {
              var r = e(n);
              throw new Error("Introspection result missing possibleTypes: " + r + ".");
            }
            return new l({
              name: n.name,
              description: n.description,
              types: function() {
                return n.possibleTypes.map(getObjectType);
              }
            });
          }(r);

         case u.ENUM:
          return function buildEnumDef(r) {
            if (!r.enumValues) {
              var t = e(r);
              throw new Error("Introspection result missing enumValues: " + t + ".");
            }
            return new c({
              name: r.name,
              description: r.description,
              values: n(r.enumValues, (function(e) {
                return e.name;
              }), (function(e) {
                return {
                  description: e.description,
                  deprecationReason: e.deprecationReason
                };
              }))
            });
          }(r);

         case u.INPUT_OBJECT:
          return function buildInputObjectDef(n) {
            if (!n.inputFields) {
              var r = e(n);
              throw new Error("Introspection result missing inputFields: " + r + ".");
            }
            return new p({
              name: n.name,
              description: n.description,
              fields: function() {
                return buildInputValueDefMap(n.inputFields);
              }
            });
          }(r);
        }
      }
      var t = e(r);
      throw new Error("Invalid or incomplete introspection result. Ensure that a full introspection query is used in order to build a client schema: " + t + ".");
    }(r);
  }));
  for (var V = 0, N = o.concat(s); V < N.length; V += 1) {
    var R = N[V];
    if (D[R.name]) {
      D[R.name] = R;
    }
  }
  var O = j.queryType ? getObjectType(j.queryType) : null;
  var F = j.mutationType ? getObjectType(j.mutationType) : null;
  var L = j.subscriptionType ? getObjectType(j.subscriptionType) : null;
  var M = j.directives ? j.directives.map((function buildDirective(n) {
    if (!n.args) {
      var r = e(n);
      throw new Error("Introspection result missing directive args: " + r + ".");
    }
    if (!n.locations) {
      var t = e(n);
      throw new Error("Introspection result missing directive locations: " + t + ".");
    }
    return new i({
      name: n.name,
      description: n.description,
      isRepeatable: n.isRepeatable,
      locations: n.locations.slice(),
      args: buildInputValueDefMap(n.args)
    });
  })) : [];
  return new t({
    description: j.description,
    query: O,
    mutation: F,
    subscription: L,
    types: Object.values(D),
    directives: M,
    assumeValid: null == E ? void 0 : E.assumeValid
  });
  function getType(e) {
    if (e.kind === u.LIST) {
      var n = e.ofType;
      if (!n) {
        throw new Error("Decorated type deeper than introspection query.");
      }
      return new v(getType(n));
    }
    if (e.kind === u.NON_NULL) {
      var r = e.ofType;
      if (!r) {
        throw new Error("Decorated type deeper than introspection query.");
      }
      var t = getType(r);
      return new w(g(t));
    }
    return getNamedType(e);
  }
  function getNamedType(n) {
    var r = n.name;
    if (!r) {
      throw new Error("Unknown type reference: " + e(n) + ".");
    }
    var t = D[r];
    if (!t) {
      throw new Error("Invalid or incomplete schema, unknown type: " + r + ". Ensure that a full introspection query is used in order to build a client schema.");
    }
    return t;
  }
  function getObjectType(e) {
    return a(getNamedType(e));
  }
  function getInterfaceType(e) {
    return T(getNamedType(e));
  }
  function buildImplementationsList(n) {
    if (null === n.interfaces && n.kind === u.INTERFACE) {
      return [];
    }
    if (!n.interfaces) {
      var r = e(n);
      throw new Error("Introspection result missing interfaces: " + r + ".");
    }
    return n.interfaces.map(getInterfaceType);
  }
  function buildFieldDefMap(r) {
    if (!r.fields) {
      throw new Error("Introspection result missing fields: " + e(r) + ".");
    }
    return n(r.fields, (function(e) {
      return e.name;
    }), buildField);
  }
  function buildField(n) {
    var r = getType(n.type);
    if (!y(r)) {
      var t = e(r);
      throw new Error("Introspection must provide output type for fields, but received: " + t + ".");
    }
    if (!n.args) {
      var i = e(n);
      throw new Error("Introspection result missing field args: " + i + ".");
    }
    return {
      description: n.description,
      deprecationReason: n.deprecationReason,
      type: r,
      args: buildInputValueDefMap(n.args)
    };
  }
  function buildInputValueDefMap(e) {
    return n(e, (function(e) {
      return e.name;
    }), buildInputValue);
  }
  function buildInputValue(n) {
    var t = getType(n.type);
    if (!b(t)) {
      var i = e(t);
      throw new Error("Introspection must provide input type for arguments, but received: " + i + ".");
    }
    var o = null != n.defaultValue ? I(r(n.defaultValue), t) : void 0;
    return {
      description: n.description,
      type: t,
      defaultValue: o,
      deprecationReason: n.deprecationReason
    };
  }
}

export { buildClientSchema };
