"use strict";

Object.defineProperty(exports, "__esModule", {
  value: !0
});

var e = require("../jsutils/keyMap.js");

var n = require("../jsutils/invariant.js");

var r = require("../jsutils/naturalCompare.js");

var a = require("../language/printer.js");

var i = require("../language/visitor.js");

var t = require("../type/scalars.js");

var s = require("../type/definition.js");

var p = require("./astFromValue.js");

var d = {
  TYPE_REMOVED: "TYPE_REMOVED",
  TYPE_CHANGED_KIND: "TYPE_CHANGED_KIND",
  TYPE_REMOVED_FROM_UNION: "TYPE_REMOVED_FROM_UNION",
  VALUE_REMOVED_FROM_ENUM: "VALUE_REMOVED_FROM_ENUM",
  REQUIRED_INPUT_FIELD_ADDED: "REQUIRED_INPUT_FIELD_ADDED",
  IMPLEMENTED_INTERFACE_REMOVED: "IMPLEMENTED_INTERFACE_REMOVED",
  FIELD_REMOVED: "FIELD_REMOVED",
  FIELD_CHANGED_KIND: "FIELD_CHANGED_KIND",
  REQUIRED_ARG_ADDED: "REQUIRED_ARG_ADDED",
  ARG_REMOVED: "ARG_REMOVED",
  ARG_CHANGED_KIND: "ARG_CHANGED_KIND",
  DIRECTIVE_REMOVED: "DIRECTIVE_REMOVED",
  DIRECTIVE_ARG_REMOVED: "DIRECTIVE_ARG_REMOVED",
  REQUIRED_DIRECTIVE_ARG_ADDED: "REQUIRED_DIRECTIVE_ARG_ADDED",
  DIRECTIVE_REPEATABLE_REMOVED: "DIRECTIVE_REPEATABLE_REMOVED",
  DIRECTIVE_LOCATION_REMOVED: "DIRECTIVE_LOCATION_REMOVED"
};

var E = {
  VALUE_ADDED_TO_ENUM: "VALUE_ADDED_TO_ENUM",
  TYPE_ADDED_TO_UNION: "TYPE_ADDED_TO_UNION",
  OPTIONAL_INPUT_FIELD_ADDED: "OPTIONAL_INPUT_FIELD_ADDED",
  OPTIONAL_ARG_ADDED: "OPTIONAL_ARG_ADDED",
  IMPLEMENTED_INTERFACE_ADDED: "IMPLEMENTED_INTERFACE_ADDED",
  ARG_DEFAULT_VALUE_CHANGE: "ARG_DEFAULT_VALUE_CHANGE"
};

function findSchemaChanges(e, n) {
  return function findTypeChanges(e, n) {
    var r = [];
    var a = diff(Object.values(e.getTypeMap()), Object.values(n.getTypeMap()));
    for (var i = 0, p = a.removed; i < p.length; i += 1) {
      r.push({
        type: d.TYPE_REMOVED,
        description: t.isSpecifiedScalarType(f = p[i]) ? "Standard scalar " + f.name + " was removed because it is not referenced anymore." : f.name + " was removed."
      });
    }
    for (var E = 0, o = a.persisted; E < o.length; E += 1) {
      var l = o[E];
      var f = l[0];
      var D = l[1];
      if (s.isEnumType(oldType$1) && s.isEnumType(D)) {
        r.push.apply(r, findEnumTypeChanges(oldType$1, D));
      } else if (s.isUnionType(oldType$1) && s.isUnionType(D)) {
        r.push.apply(r, findUnionTypeChanges(oldType$1, D));
      } else if (s.isInputObjectType(oldType$1) && s.isInputObjectType(D)) {
        r.push.apply(r, findInputObjectTypeChanges(oldType$1, D));
      } else if (s.isObjectType(oldType$1) && s.isObjectType(D)) {
        r.push.apply(r, findFieldChanges(oldType$1, D).concat(findImplementedInterfacesChanges(oldType$1, D)));
      } else if (s.isInterfaceType(oldType$1) && s.isInterfaceType(D)) {
        r.push.apply(r, findFieldChanges(oldType$1, D).concat(findImplementedInterfacesChanges(oldType$1, D)));
      } else if (oldType$1.constructor !== D.constructor) {
        r.push({
          type: d.TYPE_CHANGED_KIND,
          description: oldType$1.name + " changed from " + typeKindName(oldType$1) + " to " + typeKindName(D) + "."
        });
      }
    }
    return r;
  }(e, n).concat(function findDirectiveChanges(e, n) {
    var r = [];
    var a = diff(e.getDirectives(), n.getDirectives());
    for (var i = 0, t = a.removed; i < t.length; i += 1) {
      r.push({
        type: d.DIRECTIVE_REMOVED,
        description: t[i].name + " was removed."
      });
    }
    for (var p = 0, E = a.persisted; p < E.length; p += 1) {
      var o = E[p];
      var l = o[1];
      var f = diff(oldDirective$1.args, l.args);
      for (var D = 0, u = f.added; D < u.length; D += 1) {
        var _ = u[D];
        if (s.isRequiredArgument(_)) {
          r.push({
            type: d.REQUIRED_DIRECTIVE_ARG_ADDED,
            description: "A required arg " + _.name + " on directive " + oldDirective$1.name + " was added."
          });
        }
      }
      for (var y = 0, c = f.removed; y < c.length; y += 1) {
        r.push({
          type: d.DIRECTIVE_ARG_REMOVED,
          description: c[y].name + " was removed from " + oldDirective$1.name + "."
        });
      }
      if (oldDirective$1.isRepeatable && !l.isRepeatable) {
        r.push({
          type: d.DIRECTIVE_REPEATABLE_REMOVED,
          description: "Repeatable flag was removed from " + oldDirective$1.name + "."
        });
      }
      for (var T = 0, g = oldDirective$1.locations; T < g.length; T += 1) {
        var v = g[T];
        if (!l.locations.includes(v)) {
          r.push({
            type: d.DIRECTIVE_LOCATION_REMOVED,
            description: v + " was removed from " + oldDirective$1.name + "."
          });
        }
      }
    }
    return r;
  }(e, n));
}

function findInputObjectTypeChanges(e, n) {
  var r = [];
  var a = diff(Object.values(e.getFields()), Object.values(n.getFields()));
  for (var i = 0, t = a.added; i < t.length; i += 1) {
    if (s.isRequiredInputField(u = t[i])) {
      r.push({
        type: d.REQUIRED_INPUT_FIELD_ADDED,
        description: "A required field " + u.name + " on input type " + e.name + " was added."
      });
    } else {
      r.push({
        type: E.OPTIONAL_INPUT_FIELD_ADDED,
        description: "An optional field " + u.name + " on input type " + e.name + " was added."
      });
    }
  }
  for (var p = 0, o = a.removed; p < o.length; p += 1) {
    r.push({
      type: d.FIELD_REMOVED,
      description: e.name + "." + o[p].name + " was removed."
    });
  }
  for (var l = 0, f = a.persisted; l < f.length; l += 1) {
    var D = f[l];
    var u = D[1];
    if (!isChangeSafeForInputObjectFieldOrFieldArg(oldField$1.type, newField$1.type)) {
      r.push({
        type: d.FIELD_CHANGED_KIND,
        description: e.name + "." + oldField$1.name + " changed type from " + String(oldField$1.type) + " to " + String(newField$1.type) + "."
      });
    }
  }
  return r;
}

function findUnionTypeChanges(e, n) {
  var r = [];
  var a = diff(e.getTypes(), n.getTypes());
  for (var i = 0, t = a.added; i < t.length; i += 1) {
    r.push({
      type: E.TYPE_ADDED_TO_UNION,
      description: t[i].name + " was added to union type " + e.name + "."
    });
  }
  for (var s = 0, p = a.removed; s < p.length; s += 1) {
    r.push({
      type: d.TYPE_REMOVED_FROM_UNION,
      description: p[s].name + " was removed from union type " + e.name + "."
    });
  }
  return r;
}

function findEnumTypeChanges(e, n) {
  var r = [];
  var a = diff(e.getValues(), n.getValues());
  for (var i = 0, t = a.added; i < t.length; i += 1) {
    r.push({
      type: E.VALUE_ADDED_TO_ENUM,
      description: t[i].name + " was added to enum type " + e.name + "."
    });
  }
  for (var s = 0, p = a.removed; s < p.length; s += 1) {
    r.push({
      type: d.VALUE_REMOVED_FROM_ENUM,
      description: p[s].name + " was removed from enum type " + e.name + "."
    });
  }
  return r;
}

function findImplementedInterfacesChanges(e, n) {
  var r = [];
  var a = diff(e.getInterfaces(), n.getInterfaces());
  for (var i = 0, t = a.added; i < t.length; i += 1) {
    r.push({
      type: E.IMPLEMENTED_INTERFACE_ADDED,
      description: t[i].name + " added to interfaces implemented by " + e.name + "."
    });
  }
  for (var s = 0, p = a.removed; s < p.length; s += 1) {
    r.push({
      type: d.IMPLEMENTED_INTERFACE_REMOVED,
      description: e.name + " no longer implements interface " + p[s].name + "."
    });
  }
  return r;
}

function findFieldChanges(e, n) {
  var r = [];
  var a = diff(Object.values(e.getFields()), Object.values(n.getFields()));
  for (var i = 0, t = a.removed; i < t.length; i += 1) {
    r.push({
      type: d.FIELD_REMOVED,
      description: e.name + "." + t[i].name + " was removed."
    });
  }
  for (var s = 0, p = a.persisted; s < p.length; s += 1) {
    var E = p[s];
    var o = E[1];
    r.push.apply(r, findArgChanges(e, oldField$1, o));
    if (!isChangeSafeForObjectOrInterfaceField(oldField$1.type, o.type)) {
      r.push({
        type: d.FIELD_CHANGED_KIND,
        description: e.name + "." + oldField$1.name + " changed type from " + String(oldField$1.type) + " to " + String(o.type) + "."
      });
    }
  }
  return r;
}

function findArgChanges(e, n, r) {
  var a = [];
  var i = diff(n.args, r.args);
  for (var t = 0, p = i.removed; t < p.length; t += 1) {
    a.push({
      type: d.ARG_REMOVED,
      description: e.name + "." + n.name + " arg " + p[t].name + " was removed."
    });
  }
  for (var o = 0, l = i.persisted; o < l.length; o += 1) {
    var f = l[o];
    var D = f[1];
    if (!isChangeSafeForInputObjectFieldOrFieldArg(oldArg$1.type, D.type)) {
      a.push({
        type: d.ARG_CHANGED_KIND,
        description: e.name + "." + n.name + " arg " + oldArg$1.name + " has changed type from " + String(oldArg$1.type) + " to " + String(D.type) + "."
      });
    } else if (void 0 !== oldArg$1.defaultValue) {
      if (void 0 === D.defaultValue) {
        a.push({
          type: E.ARG_DEFAULT_VALUE_CHANGE,
          description: e.name + "." + n.name + " arg " + oldArg$1.name + " defaultValue was removed."
        });
      } else {
        var u = stringifyValue(oldArg$1.defaultValue, oldArg$1.type);
        var _ = stringifyValue(D.defaultValue, D.type);
        if (u !== _) {
          a.push({
            type: E.ARG_DEFAULT_VALUE_CHANGE,
            description: e.name + "." + n.name + " arg " + oldArg$1.name + " has changed defaultValue from " + u + " to " + _ + "."
          });
        }
      }
    }
  }
  for (var y = 0, c = i.added; y < c.length; y += 1) {
    var T = c[y];
    if (s.isRequiredArgument(T)) {
      a.push({
        type: d.REQUIRED_ARG_ADDED,
        description: "A required arg " + T.name + " on " + e.name + "." + n.name + " was added."
      });
    } else {
      a.push({
        type: E.OPTIONAL_ARG_ADDED,
        description: "An optional arg " + T.name + " on " + e.name + "." + n.name + " was added."
      });
    }
  }
  return a;
}

function isChangeSafeForObjectOrInterfaceField(e, n) {
  if (s.isListType(e)) {
    return s.isListType(n) && isChangeSafeForObjectOrInterfaceField(e.ofType, n.ofType) || s.isNonNullType(n) && isChangeSafeForObjectOrInterfaceField(e, n.ofType);
  }
  if (s.isNonNullType(e)) {
    return s.isNonNullType(n) && isChangeSafeForObjectOrInterfaceField(e.ofType, n.ofType);
  }
  return s.isNamedType(n) && e.name === n.name || s.isNonNullType(n) && isChangeSafeForObjectOrInterfaceField(e, n.ofType);
}

function isChangeSafeForInputObjectFieldOrFieldArg(e, n) {
  if (s.isListType(e)) {
    return s.isListType(n) && isChangeSafeForInputObjectFieldOrFieldArg(e.ofType, n.ofType);
  }
  if (s.isNonNullType(e)) {
    return s.isNonNullType(n) && isChangeSafeForInputObjectFieldOrFieldArg(e.ofType, n.ofType) || !s.isNonNullType(n) && isChangeSafeForInputObjectFieldOrFieldArg(e.ofType, n);
  }
  return s.isNamedType(n) && e.name === n.name;
}

function typeKindName(e) {
  if (s.isScalarType(e)) {
    return "a Scalar type";
  }
  if (s.isObjectType(e)) {
    return "an Object type";
  }
  if (s.isInterfaceType(e)) {
    return "an Interface type";
  }
  if (s.isUnionType(e)) {
    return "a Union type";
  }
  if (s.isEnumType(e)) {
    return "an Enum type";
  }
  if (s.isInputObjectType(e)) {
    return "an Input type";
  }
  n.invariant(!1);
}

function stringifyValue(e, t) {
  var s = p.astFromValue(e, t);
  null != s || n.invariant(!1);
  var d = i.visit(s, {
    ObjectValue: function ObjectValue(e) {
      var n = [].concat(e.fields);
      n.sort((function(e, n) {
        return r.naturalCompare(e.name.value, n.name.value);
      }));
      return Object.assign({}, e, {
        fields: n
      });
    }
  });
  return a.print(d);
}

function diff(n, r) {
  var a = [];
  var i = [];
  var t = [];
  var s = e.keyMap(n, (function(e) {
    return e.name;
  }));
  var p = e.keyMap(r, (function(e) {
    return e.name;
  }));
  for (var d = 0, E = n; d < E.length; d += 1) {
    var o = E[d];
    var l = p[o.name];
    if (void 0 === l) {
      i.push(o);
    } else {
      t.push([ o, l ]);
    }
  }
  for (var f = 0, D = r; f < D.length; f += 1) {
    var u = D[f];
    if (void 0 === s[u.name]) {
      a.push(u);
    }
  }
  return {
    added: a,
    persisted: t,
    removed: i
  };
}

exports.BreakingChangeType = d;

exports.DangerousChangeType = E;

exports.findBreakingChanges = function findBreakingChanges(e, n) {
  return findSchemaChanges(e, n).filter((function(e) {
    return e.type in d;
  }));
};

exports.findDangerousChanges = function findDangerousChanges(e, n) {
  return findSchemaChanges(e, n).filter((function(e) {
    return e.type in E;
  }));
};
