import { keyMap as e } from "../jsutils/keyMap.mjs";

import { invariant as n } from "../jsutils/invariant.mjs";

import { naturalCompare as r } from "../jsutils/naturalCompare.mjs";

import { print as a } from "../language/printer.mjs";

import { visit as t } from "../language/visitor.mjs";

import { isSpecifiedScalarType as i } from "../type/scalars.mjs";

import { isRequiredArgument as d, isEnumType as E, isUnionType as o, isInputObjectType as p, isObjectType as s, isInterfaceType as f, isRequiredInputField as D, isListType as l, isNonNullType as _, isNamedType as u, isScalarType as m } from "../type/definition.mjs";

import { astFromValue as c } from "./astFromValue.mjs";

var g = {
  TYPE_REMOVED: "TYPE_REMOVED",
  TYPE_CHANGED_KIND: "TYPE_CHANGED_KIND",
  TYPE_REMOVED_FROM_UNION: "TYPE_REMOVED_FROM_UNION",
  VALUE_REMOVED_FROM_ENUM: "VALUE_REMOVED_FROM_ENUM",
  REQUIRED_INPUT_FIELD_ADDED: "REQUIRED_INPUT_FIELD_ADDED",
  IMPLEMENTED_INTERFACE_REMOVED: "IMPLEMENTED_INTERFACE_REMOVED",
  FIELD_REMOVED: "FIELD_REMOVED",
  FIELD_CHANGED_KIND: "FIELD_CHANGED_KIND",
  REQUIRED_ARG_ADDED: "REQUIRED_ARG_ADDED",
  ARG_REMOVED: "ARG_REMOVED",
  ARG_CHANGED_KIND: "ARG_CHANGED_KIND",
  DIRECTIVE_REMOVED: "DIRECTIVE_REMOVED",
  DIRECTIVE_ARG_REMOVED: "DIRECTIVE_ARG_REMOVED",
  REQUIRED_DIRECTIVE_ARG_ADDED: "REQUIRED_DIRECTIVE_ARG_ADDED",
  DIRECTIVE_REPEATABLE_REMOVED: "DIRECTIVE_REPEATABLE_REMOVED",
  DIRECTIVE_LOCATION_REMOVED: "DIRECTIVE_LOCATION_REMOVED"
};

var I = {
  VALUE_ADDED_TO_ENUM: "VALUE_ADDED_TO_ENUM",
  TYPE_ADDED_TO_UNION: "TYPE_ADDED_TO_UNION",
  OPTIONAL_INPUT_FIELD_ADDED: "OPTIONAL_INPUT_FIELD_ADDED",
  OPTIONAL_ARG_ADDED: "OPTIONAL_ARG_ADDED",
  IMPLEMENTED_INTERFACE_ADDED: "IMPLEMENTED_INTERFACE_ADDED",
  ARG_DEFAULT_VALUE_CHANGE: "ARG_DEFAULT_VALUE_CHANGE"
};

function findBreakingChanges(e, n) {
  return findSchemaChanges(e, n).filter((function(e) {
    return e.type in g;
  }));
}

function findDangerousChanges(e, n) {
  return findSchemaChanges(e, n).filter((function(e) {
    return e.type in I;
  }));
}

function findSchemaChanges(e, n) {
  return function findTypeChanges(e, n) {
    var r = [];
    var a = diff(Object.values(e.getTypeMap()), Object.values(n.getTypeMap()));
    for (var t = 0, d = a.removed; t < d.length; t += 1) {
      r.push({
        type: g.TYPE_REMOVED,
        description: i(u = d[t]) ? "Standard scalar " + u.name + " was removed because it is not referenced anymore." : u.name + " was removed."
      });
    }
    for (var D = 0, l = a.persisted; D < l.length; D += 1) {
      var _ = l[D];
      var u = _[0];
      var m = _[1];
      if (E(oldType$1) && E(m)) {
        r.push.apply(r, findEnumTypeChanges(oldType$1, m));
      } else if (o(oldType$1) && o(m)) {
        r.push.apply(r, findUnionTypeChanges(oldType$1, m));
      } else if (p(oldType$1) && p(m)) {
        r.push.apply(r, findInputObjectTypeChanges(oldType$1, m));
      } else if (s(oldType$1) && s(m)) {
        r.push.apply(r, findFieldChanges(oldType$1, m).concat(findImplementedInterfacesChanges(oldType$1, m)));
      } else if (f(oldType$1) && f(m)) {
        r.push.apply(r, findFieldChanges(oldType$1, m).concat(findImplementedInterfacesChanges(oldType$1, m)));
      } else if (oldType$1.constructor !== m.constructor) {
        r.push({
          type: g.TYPE_CHANGED_KIND,
          description: oldType$1.name + " changed from " + typeKindName(oldType$1) + " to " + typeKindName(m) + "."
        });
      }
    }
    return r;
  }(e, n).concat(function findDirectiveChanges(e, n) {
    var r = [];
    var a = diff(e.getDirectives(), n.getDirectives());
    for (var t = 0, i = a.removed; t < i.length; t += 1) {
      r.push({
        type: g.DIRECTIVE_REMOVED,
        description: i[t].name + " was removed."
      });
    }
    for (var E = 0, o = a.persisted; E < o.length; E += 1) {
      var p = o[E];
      var s = p[1];
      var f = diff(oldDirective$1.args, s.args);
      for (var D = 0, l = f.added; D < l.length; D += 1) {
        var _ = l[D];
        if (d(_)) {
          r.push({
            type: g.REQUIRED_DIRECTIVE_ARG_ADDED,
            description: "A required arg " + _.name + " on directive " + oldDirective$1.name + " was added."
          });
        }
      }
      for (var u = 0, m = f.removed; u < m.length; u += 1) {
        r.push({
          type: g.DIRECTIVE_ARG_REMOVED,
          description: m[u].name + " was removed from " + oldDirective$1.name + "."
        });
      }
      if (oldDirective$1.isRepeatable && !s.isRepeatable) {
        r.push({
          type: g.DIRECTIVE_REPEATABLE_REMOVED,
          description: "Repeatable flag was removed from " + oldDirective$1.name + "."
        });
      }
      for (var c = 0, I = oldDirective$1.locations; c < I.length; c += 1) {
        var v = I[c];
        if (!s.locations.includes(v)) {
          r.push({
            type: g.DIRECTIVE_LOCATION_REMOVED,
            description: v + " was removed from " + oldDirective$1.name + "."
          });
        }
      }
    }
    return r;
  }(e, n));
}

function findInputObjectTypeChanges(e, n) {
  var r = [];
  var a = diff(Object.values(e.getFields()), Object.values(n.getFields()));
  for (var t = 0, i = a.added; t < i.length; t += 1) {
    if (D(f = i[t])) {
      r.push({
        type: g.REQUIRED_INPUT_FIELD_ADDED,
        description: "A required field " + f.name + " on input type " + e.name + " was added."
      });
    } else {
      r.push({
        type: I.OPTIONAL_INPUT_FIELD_ADDED,
        description: "An optional field " + f.name + " on input type " + e.name + " was added."
      });
    }
  }
  for (var d = 0, E = a.removed; d < E.length; d += 1) {
    r.push({
      type: g.FIELD_REMOVED,
      description: e.name + "." + E[d].name + " was removed."
    });
  }
  for (var o = 0, p = a.persisted; o < p.length; o += 1) {
    var s = p[o];
    var f = s[1];
    if (!isChangeSafeForInputObjectFieldOrFieldArg(oldField$1.type, newField$1.type)) {
      r.push({
        type: g.FIELD_CHANGED_KIND,
        description: e.name + "." + oldField$1.name + " changed type from " + String(oldField$1.type) + " to " + String(newField$1.type) + "."
      });
    }
  }
  return r;
}

function findUnionTypeChanges(e, n) {
  var r = [];
  var a = diff(e.getTypes(), n.getTypes());
  for (var t = 0, i = a.added; t < i.length; t += 1) {
    r.push({
      type: I.TYPE_ADDED_TO_UNION,
      description: i[t].name + " was added to union type " + e.name + "."
    });
  }
  for (var d = 0, E = a.removed; d < E.length; d += 1) {
    r.push({
      type: g.TYPE_REMOVED_FROM_UNION,
      description: E[d].name + " was removed from union type " + e.name + "."
    });
  }
  return r;
}

function findEnumTypeChanges(e, n) {
  var r = [];
  var a = diff(e.getValues(), n.getValues());
  for (var t = 0, i = a.added; t < i.length; t += 1) {
    r.push({
      type: I.VALUE_ADDED_TO_ENUM,
      description: i[t].name + " was added to enum type " + e.name + "."
    });
  }
  for (var d = 0, E = a.removed; d < E.length; d += 1) {
    r.push({
      type: g.VALUE_REMOVED_FROM_ENUM,
      description: E[d].name + " was removed from enum type " + e.name + "."
    });
  }
  return r;
}

function findImplementedInterfacesChanges(e, n) {
  var r = [];
  var a = diff(e.getInterfaces(), n.getInterfaces());
  for (var t = 0, i = a.added; t < i.length; t += 1) {
    r.push({
      type: I.IMPLEMENTED_INTERFACE_ADDED,
      description: i[t].name + " added to interfaces implemented by " + e.name + "."
    });
  }
  for (var d = 0, E = a.removed; d < E.length; d += 1) {
    r.push({
      type: g.IMPLEMENTED_INTERFACE_REMOVED,
      description: e.name + " no longer implements interface " + E[d].name + "."
    });
  }
  return r;
}

function findFieldChanges(e, n) {
  var r = [];
  var a = diff(Object.values(e.getFields()), Object.values(n.getFields()));
  for (var t = 0, i = a.removed; t < i.length; t += 1) {
    r.push({
      type: g.FIELD_REMOVED,
      description: e.name + "." + i[t].name + " was removed."
    });
  }
  for (var d = 0, E = a.persisted; d < E.length; d += 1) {
    var o = E[d];
    var p = o[1];
    r.push.apply(r, findArgChanges(e, oldField$1, p));
    if (!isChangeSafeForObjectOrInterfaceField(oldField$1.type, p.type)) {
      r.push({
        type: g.FIELD_CHANGED_KIND,
        description: e.name + "." + oldField$1.name + " changed type from " + String(oldField$1.type) + " to " + String(p.type) + "."
      });
    }
  }
  return r;
}

function findArgChanges(e, n, r) {
  var a = [];
  var t = diff(n.args, r.args);
  for (var i = 0, E = t.removed; i < E.length; i += 1) {
    a.push({
      type: g.ARG_REMOVED,
      description: e.name + "." + n.name + " arg " + E[i].name + " was removed."
    });
  }
  for (var o = 0, p = t.persisted; o < p.length; o += 1) {
    var s = p[o];
    var f = s[1];
    if (!isChangeSafeForInputObjectFieldOrFieldArg(oldArg$1.type, f.type)) {
      a.push({
        type: g.ARG_CHANGED_KIND,
        description: e.name + "." + n.name + " arg " + oldArg$1.name + " has changed type from " + String(oldArg$1.type) + " to " + String(f.type) + "."
      });
    } else if (void 0 !== oldArg$1.defaultValue) {
      if (void 0 === f.defaultValue) {
        a.push({
          type: I.ARG_DEFAULT_VALUE_CHANGE,
          description: e.name + "." + n.name + " arg " + oldArg$1.name + " defaultValue was removed."
        });
      } else {
        var D = stringifyValue(oldArg$1.defaultValue, oldArg$1.type);
        var l = stringifyValue(f.defaultValue, f.type);
        if (D !== l) {
          a.push({
            type: I.ARG_DEFAULT_VALUE_CHANGE,
            description: e.name + "." + n.name + " arg " + oldArg$1.name + " has changed defaultValue from " + D + " to " + l + "."
          });
        }
      }
    }
  }
  for (var _ = 0, u = t.added; _ < u.length; _ += 1) {
    var m = u[_];
    if (d(m)) {
      a.push({
        type: g.REQUIRED_ARG_ADDED,
        description: "A required arg " + m.name + " on " + e.name + "." + n.name + " was added."
      });
    } else {
      a.push({
        type: I.OPTIONAL_ARG_ADDED,
        description: "An optional arg " + m.name + " on " + e.name + "." + n.name + " was added."
      });
    }
  }
  return a;
}

function isChangeSafeForObjectOrInterfaceField(e, n) {
  if (l(e)) {
    return l(n) && isChangeSafeForObjectOrInterfaceField(e.ofType, n.ofType) || _(n) && isChangeSafeForObjectOrInterfaceField(e, n.ofType);
  }
  if (_(e)) {
    return _(n) && isChangeSafeForObjectOrInterfaceField(e.ofType, n.ofType);
  }
  return u(n) && e.name === n.name || _(n) && isChangeSafeForObjectOrInterfaceField(e, n.ofType);
}

function isChangeSafeForInputObjectFieldOrFieldArg(e, n) {
  if (l(e)) {
    return l(n) && isChangeSafeForInputObjectFieldOrFieldArg(e.ofType, n.ofType);
  }
  if (_(e)) {
    return _(n) && isChangeSafeForInputObjectFieldOrFieldArg(e.ofType, n.ofType) || !_(n) && isChangeSafeForInputObjectFieldOrFieldArg(e.ofType, n);
  }
  return u(n) && e.name === n.name;
}

function typeKindName(e) {
  if (m(e)) {
    return "a Scalar type";
  }
  if (s(e)) {
    return "an Object type";
  }
  if (f(e)) {
    return "an Interface type";
  }
  if (o(e)) {
    return "a Union type";
  }
  if (E(e)) {
    return "an Enum type";
  }
  if (p(e)) {
    return "an Input type";
  }
  n(!1);
}

function stringifyValue(e, i) {
  var d = c(e, i);
  null != d || n(!1);
  var E = t(d, {
    ObjectValue: function ObjectValue(e) {
      var n = [].concat(e.fields);
      n.sort((function(e, n) {
        return r(e.name.value, n.name.value);
      }));
      return Object.assign({}, e, {
        fields: n
      });
    }
  });
  return a(E);
}

function diff(n, r) {
  var a = [];
  var t = [];
  var i = [];
  var d = e(n, (function(e) {
    return e.name;
  }));
  var E = e(r, (function(e) {
    return e.name;
  }));
  for (var o = 0, p = n; o < p.length; o += 1) {
    var s = p[o];
    var f = E[s.name];
    if (void 0 === f) {
      t.push(s);
    } else {
      i.push([ s, f ]);
    }
  }
  for (var D = 0, l = r; D < l.length; D += 1) {
    var _ = l[D];
    if (void 0 === d[_.name]) {
      a.push(_);
    }
  }
  return {
    added: a,
    persisted: i,
    removed: t
  };
}

export { g as BreakingChangeType, I as DangerousChangeType, findBreakingChanges, findDangerousChanges };
