"use strict";

Object.defineProperty(exports, "__esModule", {
  value: !0
});

var t = require("../language/kinds.js");

var e = require("../language/visitor.js");

var n = require("../utilities/TypeInfo.js");

var r = function ASTValidationContext(t, e) {
  this._ast = t;
  this._fragments = void 0;
  this._fragmentSpreads = new Map;
  this._recursivelyReferencedFragments = new Map;
  this._onError = e;
};

r.prototype.reportError = function reportError(t) {
  this._onError(t);
};

r.prototype.getDocument = function getDocument() {
  return this._ast;
};

r.prototype.getFragment = function getFragment(e) {
  var n;
  if (this._fragments) {
    n = this._fragments;
  } else {
    n = Object.create(null);
    for (var r = 0, i = this.getDocument().definitions; r < i.length; r += 1) {
      var a = i[r];
      if (a.kind === t.Kind.FRAGMENT_DEFINITION) {
        n[a.name.value] = a;
      }
    }
    this._fragments = n;
  }
  return n[e];
};

r.prototype.getFragmentSpreads = function getFragmentSpreads(e) {
  var n = this._fragmentSpreads.get(e);
  if (!n) {
    n = [];
    var r = [ e ];
    var i;
    while (i = r.pop()) {
      for (var a = 0, o = i.selections; a < o.length; a += 1) {
        var s = o[a];
        if (s.kind === t.Kind.FRAGMENT_SPREAD) {
          n.push(s);
        } else if (s.selectionSet) {
          r.push(s.selectionSet);
        }
      }
    }
    this._fragmentSpreads.set(e, n);
  }
  return n;
};

r.prototype.getRecursivelyReferencedFragments = function getRecursivelyReferencedFragments(t) {
  var e = this._recursivelyReferencedFragments.get(t);
  if (!e) {
    e = [];
    var n = Object.create(null);
    var r = [ t.selectionSet ];
    var i;
    while (i = r.pop()) {
      for (var a = 0, o = this.getFragmentSpreads(i); a < o.length; a += 1) {
        var s = o[a].name.value;
        if (!0 !== n[s]) {
          n[s] = !0;
          var p = this.getFragment(s);
          if (p) {
            e.push(p);
            r.push(p.selectionSet);
          }
        }
      }
    }
    this._recursivelyReferencedFragments.set(t, e);
  }
  return e;
};

var i = function(t) {
  function SDLValidationContext(e, n, r) {
    t.call(this, e, r);
    this._schema = n;
  }
  if (t) {
    SDLValidationContext.__proto__ = t;
  }
  (SDLValidationContext.prototype = Object.create(t && t.prototype)).constructor = SDLValidationContext;
  SDLValidationContext.prototype.getSchema = function getSchema() {
    return this._schema;
  };
  return SDLValidationContext;
}(r);

var a = function(t) {
  function ValidationContext(e, n, r, i) {
    t.call(this, n, i);
    this._schema = e;
    this._typeInfo = r;
    this._variableUsages = new Map;
    this._recursiveVariableUsages = new Map;
  }
  if (t) {
    ValidationContext.__proto__ = t;
  }
  (ValidationContext.prototype = Object.create(t && t.prototype)).constructor = ValidationContext;
  ValidationContext.prototype.getSchema = function getSchema() {
    return this._schema;
  };
  ValidationContext.prototype.getVariableUsages = function getVariableUsages(t) {
    var r = this._variableUsages.get(t);
    if (!r) {
      var i = [];
      var a = new n.TypeInfo(this._schema);
      e.visit(t, n.visitWithTypeInfo(a, {
        VariableDefinition: function() {
          return !1;
        },
        Variable: function Variable(t) {
          i.push({
            node: t,
            type: a.getInputType(),
            defaultValue: a.getDefaultValue()
          });
        }
      }));
      this._variableUsages.set(t, r = i);
    }
    return r;
  };
  ValidationContext.prototype.getRecursiveVariableUsages = function getRecursiveVariableUsages(t) {
    var e = this._recursiveVariableUsages.get(t);
    if (!e) {
      e = this.getVariableUsages(t);
      for (var n = 0, r = this.getRecursivelyReferencedFragments(t); n < r.length; n += 1) {
        e = e.concat(this.getVariableUsages(r[n]));
      }
      this._recursiveVariableUsages.set(t, e);
    }
    return e;
  };
  ValidationContext.prototype.getType = function getType() {
    return this._typeInfo.getType();
  };
  ValidationContext.prototype.getParentType = function getParentType() {
    return this._typeInfo.getParentType();
  };
  ValidationContext.prototype.getInputType = function getInputType() {
    return this._typeInfo.getInputType();
  };
  ValidationContext.prototype.getParentInputType = function getParentInputType() {
    return this._typeInfo.getParentInputType();
  };
  ValidationContext.prototype.getFieldDef = function getFieldDef() {
    return this._typeInfo.getFieldDef();
  };
  ValidationContext.prototype.getDirective = function getDirective() {
    return this._typeInfo.getDirective();
  };
  ValidationContext.prototype.getArgument = function getArgument() {
    return this._typeInfo.getArgument();
  };
  ValidationContext.prototype.getEnumValue = function getEnumValue() {
    return this._typeInfo.getEnumValue();
  };
  return ValidationContext;
}(r);

exports.SDLValidationContext = i;

exports.ValidationContext = a;
