import { invariant as e } from "../../jsutils/invariant.mjs";

import { GraphQLError as r } from "../../error/GraphQLError.mjs";

import { Kind as E } from "../../language/kinds.mjs";

import { DirectiveLocation as I } from "../../language/directiveLocation.mjs";

import { specifiedDirectives as N } from "../../type/directives.mjs";

function KnownDirectivesRule(n) {
  var t = Object.create(null);
  var T = n.getSchema();
  for (var a = 0, i = T ? T.getDirectives() : N; a < i.length; a += 1) {
    var c = i[a];
    t[c.name] = c.locations;
  }
  for (var _ = 0, o = n.getDocument().definitions; _ < o.length; _ += 1) {
    var s = o[_];
    if (s.kind === E.DIRECTIVE_DEFINITION) {
      t[s.name.value] = s.locations.map((function(e) {
        return e.value;
      }));
    }
  }
  return {
    Directive: function Directive(N, T, a, i, c) {
      var _ = N.name.value;
      var o = t[_];
      if (!o) {
        n.reportError(new r('Unknown directive "@' + _ + '".', N));
        return;
      }
      var s = function getDirectiveLocationForASTPath(r) {
        var N = r[r.length - 1];
        "kind" in N || e(!1);
        switch (N.kind) {
         case E.OPERATION_DEFINITION:
          return function getDirectiveLocationForOperation(r) {
            switch (r) {
             case "query":
              return I.QUERY;

             case "mutation":
              return I.MUTATION;

             case "subscription":
              return I.SUBSCRIPTION;
            }
            e(!1);
          }(N.operation);

         case E.FIELD:
          return I.FIELD;

         case E.FRAGMENT_SPREAD:
          return I.FRAGMENT_SPREAD;

         case E.INLINE_FRAGMENT:
          return I.INLINE_FRAGMENT;

         case E.FRAGMENT_DEFINITION:
          return I.FRAGMENT_DEFINITION;

         case E.VARIABLE_DEFINITION:
          return I.VARIABLE_DEFINITION;

         case E.SCHEMA_DEFINITION:
         case E.SCHEMA_EXTENSION:
          return I.SCHEMA;

         case E.SCALAR_TYPE_DEFINITION:
         case E.SCALAR_TYPE_EXTENSION:
          return I.SCALAR;

         case E.OBJECT_TYPE_DEFINITION:
         case E.OBJECT_TYPE_EXTENSION:
          return I.OBJECT;

         case E.FIELD_DEFINITION:
          return I.FIELD_DEFINITION;

         case E.INTERFACE_TYPE_DEFINITION:
         case E.INTERFACE_TYPE_EXTENSION:
          return I.INTERFACE;

         case E.UNION_TYPE_DEFINITION:
         case E.UNION_TYPE_EXTENSION:
          return I.UNION;

         case E.ENUM_TYPE_DEFINITION:
         case E.ENUM_TYPE_EXTENSION:
          return I.ENUM;

         case E.ENUM_VALUE_DEFINITION:
          return I.ENUM_VALUE;

         case E.INPUT_OBJECT_TYPE_DEFINITION:
         case E.INPUT_OBJECT_TYPE_EXTENSION:
          return I.INPUT_OBJECT;

         case E.INPUT_VALUE_DEFINITION:
          var n = r[r.length - 3];
          "kind" in n || e(!1);
          return n.kind === E.INPUT_OBJECT_TYPE_DEFINITION ? I.INPUT_FIELD_DEFINITION : I.ARGUMENT_DEFINITION;
        }
      }(c);
      if (s && !o.includes(s)) {
        n.reportError(new r('Directive "@' + _ + '" may not be used on ' + s + ".", N));
      }
    }
  };
}

export { KnownDirectivesRule };
