import { inspect as e } from "../../jsutils/inspect.mjs";

import { GraphQLError as t } from "../../error/GraphQLError.mjs";

import { Kind as n } from "../../language/kinds.mjs";

import { print as r } from "../../language/printer.mjs";

import { isObjectType as a, getNamedType as i, isListType as l, isNonNullType as o, isLeafType as f, isInterfaceType as s } from "../../type/definition.mjs";

import { typeFromAST as c } from "../../utilities/typeFromAST.mjs";

function reasonMessage(e) {
  if (Array.isArray(e)) {
    return e.map((function(e) {
      return 'subfields "' + e[0] + '" conflict because ' + reasonMessage(e[1]);
    })).join(" and ");
  }
  return e;
}

function OverlappingFieldsCanBeMergedRule(e) {
  var n = new v;
  var r = new Map;
  return {
    SelectionSet: function SelectionSet(a) {
      var i = function findConflictsWithinSelectionSet(e, t, n, r, a) {
        var i = [];
        var l = getFieldsAndFragmentNames(e, t, r, a);
        var o = l[0];
        var f = l[1];
        !function collectConflictsWithin(e, t, n, r, a) {
          for (var i = 0, l = Object.entries(a); i < l.length; i += 1) {
            var o = l[i];
            var f = o[0];
            var s = o[1];
            if (s.length > 1) {
              for (var c = 0; c < s.length; c++) {
                for (var v = c + 1; v < s.length; v++) {
                  var u = findConflict(e, n, r, !1, f, s[c], s[v]);
                  if (u) {
                    t.push(u);
                  }
                }
              }
            }
          }
        }(e, i, t, n, o);
        if (0 !== f.length) {
          for (var s = 0; s < f.length; s++) {
            collectConflictsBetweenFieldsAndFragment(e, i, t, n, !1, o, f[s]);
            for (var c = s + 1; c < f.length; c++) {
              collectConflictsBetweenFragments(e, i, t, n, !1, f[s], f[c]);
            }
          }
        }
        return i;
      }(e, r, n, e.getParentType(), a);
      for (var l = 0, o = i; l < o.length; l += 1) {
        var f = o[l];
        var s = f[0];
        var c = s[0];
        var v = f[1];
        var u = f[2];
        var d = reasonMessage(s[1]);
        e.reportError(new t('Fields "' + c + '" conflict because ' + d + ". Use different aliases on the fields to fetch both if this was intentional.", v.concat(u)));
      }
    }
  };
}

function collectConflictsBetweenFieldsAndFragment(e, t, n, r, a, i, l) {
  var o = e.getFragment(l);
  if (!o) {
    return;
  }
  var f = getReferencedFieldsAndFragmentNames(e, n, o);
  var s = f[0];
  var c = f[1];
  if (i === s) {
    return;
  }
  collectConflictsBetween(e, t, n, r, a, i, s);
  for (var v = 0; v < c.length; v++) {
    collectConflictsBetweenFieldsAndFragment(e, t, n, r, a, i, c[v]);
  }
}

function collectConflictsBetweenFragments(e, t, n, r, a, i, l) {
  if (i === l) {
    return;
  }
  if (r.has(i, l, a)) {
    return;
  }
  r.add(i, l, a);
  var o = e.getFragment(i);
  var f = e.getFragment(l);
  if (!o || !f) {
    return;
  }
  var s = getReferencedFieldsAndFragmentNames(e, n, o);
  var c = s[0];
  var v = s[1];
  var u = getReferencedFieldsAndFragmentNames(e, n, f);
  var d = u[1];
  collectConflictsBetween(e, t, n, r, a, c, u[0]);
  for (var g = 0; g < d.length; g++) {
    collectConflictsBetweenFragments(e, t, n, r, a, i, d[g]);
  }
  for (var m = 0; m < v.length; m++) {
    collectConflictsBetweenFragments(e, t, n, r, a, v[m], l);
  }
}

function collectConflictsBetween(e, t, n, r, a, i, l) {
  for (var o = 0, f = Object.entries(i); o < f.length; o += 1) {
    var s = f[o];
    var c = s[0];
    var v = s[1];
    var u = l[c];
    if (u) {
      for (var d = 0; d < v.length; d++) {
        for (var g = 0; g < u.length; g++) {
          var m = findConflict(e, n, r, a, c, v[d], u[g]);
          if (m) {
            t.push(m);
          }
        }
      }
    }
  }
}

function findConflict(t, n, l, o, f, s, c) {
  var v = s[0];
  var u = s[1];
  var d = s[2];
  var g = c[0];
  var m = c[1];
  var F = c[2];
  var h = o || v !== g && a(v) && a(g);
  if (!h) {
    var p, C;
    var y = u.name.value;
    var A = m.name.value;
    if (y !== A) {
      return [ [ f, '"' + y + '" and "' + A + '" are different fields' ], [ u ], [ m ] ];
    }
    if (!function sameArguments(e, t) {
      if (e.length !== t.length) {
        return !1;
      }
      return e.every((function(e) {
        var n = t.find((function(t) {
          return t.name.value === e.name.value;
        }));
        if (!n) {
          return !1;
        }
        return function sameValue(e, t) {
          return r(e) === r(t);
        }(e.value, n.value);
      }));
    }(null !== (p = u.arguments) && void 0 !== p ? p : [], null !== (C = m.arguments) && void 0 !== C ? C : [])) {
      return [ [ f, "they have differing arguments" ], [ u ], [ m ] ];
    }
  }
  var w = null == d ? void 0 : d.type;
  var S = null == F ? void 0 : F.type;
  if (w && S && doTypesConflict(w, S)) {
    return [ [ f, 'they return conflicting types "' + e(w) + '" and "' + e(S) + '"' ], [ u ], [ m ] ];
  }
  var B = u.selectionSet;
  var N = m.selectionSet;
  if (B && N) {
    return function subfieldConflicts(e, t, n, r) {
      if (e.length > 0) {
        return [ [ t, e.map((function(e) {
          return e[0];
        })) ], [ n ].concat(e.map((function(e) {
          return e[1];
        })).flat()), [ r ].concat(e.map((function(e) {
          return e[2];
        })).flat()) ];
      }
    }(function findConflictsBetweenSubSelectionSets(e, t, n, r, a, i, l, o) {
      var f = [];
      var s = getFieldsAndFragmentNames(e, t, a, i);
      var c = s[0];
      var v = s[1];
      var u = getFieldsAndFragmentNames(e, t, l, o);
      var d = u[0];
      var g = u[1];
      collectConflictsBetween(e, f, t, n, r, c, d);
      if (0 !== g.length) {
        for (var m = 0; m < g.length; m++) {
          collectConflictsBetweenFieldsAndFragment(e, f, t, n, r, c, g[m]);
        }
      }
      if (0 !== v.length) {
        for (var F = 0; F < v.length; F++) {
          collectConflictsBetweenFieldsAndFragment(e, f, t, n, r, d, v[F]);
        }
      }
      for (var h = 0; h < v.length; h++) {
        for (var p = 0; p < g.length; p++) {
          collectConflictsBetweenFragments(e, f, t, n, r, v[h], g[p]);
        }
      }
      return f;
    }(t, n, l, h, i(w), B, i(S), N), f, u, m);
  }
}

function doTypesConflict(e, t) {
  if (l(e)) {
    return l(t) ? doTypesConflict(e.ofType, t.ofType) : !0;
  }
  if (l(t)) {
    return !0;
  }
  if (o(e)) {
    return o(t) ? doTypesConflict(e.ofType, t.ofType) : !0;
  }
  if (o(t)) {
    return !0;
  }
  if (f(e) || f(t)) {
    return e !== t;
  }
  return !1;
}

function getFieldsAndFragmentNames(e, t, n, r) {
  var a = t.get(r);
  if (a) {
    return a;
  }
  var i = Object.create(null);
  var l = Object.create(null);
  _collectFieldsAndFragmentNames(e, n, r, i, l);
  var o = [ i, Object.keys(l) ];
  t.set(r, o);
  return o;
}

function getReferencedFieldsAndFragmentNames(e, t, n) {
  var r = t.get(n.selectionSet);
  if (r) {
    return r;
  }
  var a = c(e.getSchema(), n.typeCondition);
  return getFieldsAndFragmentNames(e, t, a, n.selectionSet);
}

function _collectFieldsAndFragmentNames(e, t, r, i, l) {
  for (var o = 0, f = r.selections; o < f.length; o += 1) {
    var v = f[o];
    switch (v.kind) {
     case n.FIELD:
      var u = v.name.value;
      var d = void 0;
      if (a(t) || s(t)) {
        d = t.getFields()[u];
      }
      var g = v.alias ? v.alias.value : u;
      if (!i[g]) {
        i[g] = [];
      }
      i[g].push([ t, v, d ]);
      break;

     case n.FRAGMENT_SPREAD:
      l[v.name.value] = !0;
      break;

     case n.INLINE_FRAGMENT:
      var m = v.typeCondition;
      var F = m ? c(e.getSchema(), m) : t;
      _collectFieldsAndFragmentNames(e, F, v.selectionSet, i, l);
    }
  }
}

var v = function PairSet() {
  this._data = new Map;
};

v.prototype.has = function has(e, t, n) {
  var r;
  var a = e < t ? [ e, t ] : [ t, e ];
  var i = a[1];
  var l = null === (r = this._data.get(a[0])) || void 0 === r ? void 0 : r.get(i);
  if (void 0 === l) {
    return !1;
  }
  return n ? !0 : n === l;
};

v.prototype.add = function add(e, t, n) {
  var r = e < t ? [ e, t ] : [ t, e ];
  var a = r[0];
  var i = r[1];
  var l = this._data.get(a);
  if (void 0 === l) {
    this._data.set(a, new Map([ [ i, n ] ]));
  } else {
    l.set(i, n);
  }
};

export { OverlappingFieldsCanBeMergedRule };
