import { invariant as e } from "../../jsutils/invariant.mjs";

import { didYouMean as E } from "../../jsutils/didYouMean.mjs";

import { suggestionList as n } from "../../jsutils/suggestionList.mjs";

import { GraphQLError as t } from "../../error/GraphQLError.mjs";

import { Kind as r } from "../../language/kinds.mjs";

import { isTypeDefinitionNode as T } from "../../language/predicates.mjs";

import { isScalarType as i, isObjectType as N, isInterfaceType as o, isUnionType as s, isEnumType as _, isInputObjectType as I } from "../../type/definition.mjs";

function PossibleTypeExtensionsRule(c) {
  var O = c.getSchema();
  var u = Object.create(null);
  for (var p = 0, P = c.getDocument().definitions; p < P.length; p += 1) {
    var m = P[p];
    if (T(m)) {
      u[m.name.value] = m;
    }
  }
  return {
    ScalarTypeExtension: checkExtension,
    ObjectTypeExtension: checkExtension,
    InterfaceTypeExtension: checkExtension,
    UnionTypeExtension: checkExtension,
    EnumTypeExtension: checkExtension,
    InputObjectTypeExtension: checkExtension
  };
  function checkExtension(T) {
    var p = T.name.value;
    var P = u[p];
    var m = null == O ? void 0 : O.getType(p);
    var f;
    if (P) {
      f = a[P.kind];
    } else if (m) {
      f = function typeToExtKind(E) {
        if (i(E)) {
          return r.SCALAR_TYPE_EXTENSION;
        }
        if (N(E)) {
          return r.OBJECT_TYPE_EXTENSION;
        }
        if (o(E)) {
          return r.INTERFACE_TYPE_EXTENSION;
        }
        if (s(E)) {
          return r.UNION_TYPE_EXTENSION;
        }
        if (_(E)) {
          return r.ENUM_TYPE_EXTENSION;
        }
        if (I(E)) {
          return r.INPUT_OBJECT_TYPE_EXTENSION;
        }
        e(!1);
      }(m);
    }
    if (f) {
      if (f !== T.kind) {
        var Y = function extensionKindToTypeName(E) {
          switch (E) {
           case r.SCALAR_TYPE_EXTENSION:
            return "scalar";

           case r.OBJECT_TYPE_EXTENSION:
            return "object";

           case r.INTERFACE_TYPE_EXTENSION:
            return "interface";

           case r.UNION_TYPE_EXTENSION:
            return "union";

           case r.ENUM_TYPE_EXTENSION:
            return "enum";

           case r.INPUT_OBJECT_TYPE_EXTENSION:
            return "input object";
          }
          e(!1);
        }(T.kind);
        c.reportError(new t("Cannot extend non-" + Y + ' type "' + p + '".', P ? [ P, T ] : T));
      }
    } else {
      var S = Object.keys(Object.assign({}, u, null == O ? void 0 : O.getTypeMap()));
      var l = n(p, S);
      c.reportError(new t('Cannot extend type "' + p + '" because it is not defined.' + E(l), T.name));
    }
  }
}

var a = {};

a[r.SCALAR_TYPE_DEFINITION] = r.SCALAR_TYPE_EXTENSION;

a[r.OBJECT_TYPE_DEFINITION] = r.OBJECT_TYPE_EXTENSION;

a[r.INTERFACE_TYPE_DEFINITION] = r.INTERFACE_TYPE_EXTENSION;

a[r.UNION_TYPE_DEFINITION] = r.UNION_TYPE_EXTENSION;

a[r.ENUM_TYPE_DEFINITION] = r.ENUM_TYPE_EXTENSION;

a[r.INPUT_OBJECT_TYPE_DEFINITION] = r.INPUT_OBJECT_TYPE_EXTENSION;

export { PossibleTypeExtensionsRule };
